@@ -83,8 +83,13 @@ __device__ static inline int __hip_move_dpp_N(int src) {
                                     bound_ctrl);
 }
 
-__device__
-static constexpr int warpSize = __AMDGCN_WAVEFRONT_SIZE;
+inline __device__ const struct final {
+  __device__
+  __attribute__((always_inline, const))
+    operator int() const noexcept {
+      return __builtin_amdgcn_wavefrontsize();
+    }
+} warpSize{};
 
 // warp vote function __all __any __ballot
 __device__
@@ -99,23 +104,20 @@ int __any(int predicate) {
     return __ockl_wfany_i32(predicate);
 }
 
-// XXX from llvm/include/llvm/IR/InstrTypes.h
-#define ICMP_NE 33
-
 __device__
 inline
 unsigned long long int __ballot(int predicate) {
-    return __builtin_amdgcn_uicmp(predicate, 0, ICMP_NE);
+    return __builtin_amdgcn_ballot_w64(predicate);
 }
 
 __device__
 inline
 unsigned long long int __ballot64(int predicate) {
-    return __builtin_amdgcn_uicmp(predicate, 0, ICMP_NE);
+    return __ballot(predicate);
 }
 
 // See amd_warp_sync_functions.h for an explanation of this preprocessor flag.
-#ifdef HIP_ENABLE_WARP_SYNC_BUILTINS
+#if !defined(HIP_DISABLE_WARP_SYNC_BUILTINS)
 // Since threads in a wave do not make independent progress, __activemask()
 // always returns the exact active mask, i.e, all active threads in the wave.
 __device__
@@ -123,9 +125,10 @@ inline
 unsigned long long __activemask() {
   return __ballot(true);
 }
-#endif // HIP_ENABLE_WARP_SYNC_BUILTINS
+#endif // HIP_DISABLE_WARP_SYNC_BUILTINS
 
 __device__ static inline unsigned int __lane_id() {
+    if (static_cast<int>(warpSize) == 32) return __builtin_amdgcn_mbcnt_lo(-1, 0);
     return  __builtin_amdgcn_mbcnt_hi(
         -1, __builtin_amdgcn_mbcnt_lo(-1, 0));
 }
@@ -155,13 +158,13 @@ __device__
 inline
 double __shfl(MAYBE_UNDEF double var, int src_lane, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
-    static_assert(sizeof(double) == sizeof(uint64_t), "");
+    static_assert(sizeof(double) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl(tmp[0], src_lane, width);
     tmp[1] = __shfl(tmp[1], src_lane, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     double tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -171,13 +174,13 @@ long __shfl(MAYBE_UNDEF long var, int src_lane, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl(tmp[0], src_lane, width);
     tmp[1] = __shfl(tmp[1], src_lane, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -190,13 +193,13 @@ inline
 unsigned long __shfl(MAYBE_UNDEF unsigned long var, int src_lane, int width = warpSize) {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long) == sizeof(__hip_uint64_t), "");
 
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl(tmp[0], src_lane, width);
     tmp[1] = __shfl(tmp[1], src_lane, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -209,13 +212,13 @@ inline
 long long __shfl(MAYBE_UNDEF long long var, int src_lane, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long long) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl(tmp[0], src_lane, width);
     tmp[1] = __shfl(tmp[1], src_lane, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -223,13 +226,13 @@ __device__
 inline
 unsigned long long __shfl(MAYBE_UNDEF unsigned long long var, int src_lane, int width = warpSize) {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long long) == sizeof(__hip_uint64_t), "");
 
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl(tmp[0], src_lane, width);
     tmp[1] = __shfl(tmp[1], src_lane, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -260,13 +263,13 @@ __device__
 inline
 double __shfl_up(MAYBE_UNDEF double var, unsigned int lane_delta, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
-    static_assert(sizeof(double) == sizeof(uint64_t), "");
+    static_assert(sizeof(double) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_up(tmp[0], lane_delta, width);
     tmp[1] = __shfl_up(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     double tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -276,13 +279,13 @@ long __shfl_up(MAYBE_UNDEF long var, unsigned int lane_delta, int width = warpSi
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_up(tmp[0], lane_delta, width);
     tmp[1] = __shfl_up(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -297,13 +300,13 @@ unsigned long __shfl_up(MAYBE_UNDEF unsigned long var, unsigned int lane_delta,
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long) == sizeof(__hip_uint64_t), "");
 
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_up(tmp[0], lane_delta, width);
     tmp[1] = __shfl_up(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -317,11 +320,11 @@ inline
 long long __shfl_up(MAYBE_UNDEF long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long long) == sizeof(__hip_uint64_t), "");
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_up(tmp[0], lane_delta, width);
     tmp[1] = __shfl_up(tmp[1], lane_delta, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -331,11 +334,11 @@ inline
 unsigned long long __shfl_up(MAYBE_UNDEF unsigned long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long long) == sizeof(__hip_uint64_t), "");
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_up(tmp[0], lane_delta, width);
     tmp[1] = __shfl_up(tmp[1], lane_delta, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -366,13 +369,13 @@ __device__
 inline
 double __shfl_down(MAYBE_UNDEF double var, unsigned int lane_delta, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
-    static_assert(sizeof(double) == sizeof(uint64_t), "");
+    static_assert(sizeof(double) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_down(tmp[0], lane_delta, width);
     tmp[1] = __shfl_down(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     double tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -382,13 +385,13 @@ long __shfl_down(MAYBE_UNDEF long var, unsigned int lane_delta, int width = warp
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_down(tmp[0], lane_delta, width);
     tmp[1] = __shfl_down(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -402,13 +405,13 @@ unsigned long __shfl_down(MAYBE_UNDEF unsigned long var, unsigned int lane_delta
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long) == sizeof(__hip_uint64_t), "");
 
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_down(tmp[0], lane_delta, width);
     tmp[1] = __shfl_down(tmp[1], lane_delta, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -421,11 +424,11 @@ inline
 long long __shfl_down(MAYBE_UNDEF long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long long) == sizeof(__hip_uint64_t), "");
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_down(tmp[0], lane_delta, width);
     tmp[1] = __shfl_down(tmp[1], lane_delta, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -434,11 +437,11 @@ inline
 unsigned long long __shfl_down(MAYBE_UNDEF unsigned long long var, unsigned int lane_delta, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long long) == sizeof(__hip_uint64_t), "");
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_down(tmp[0], lane_delta, width);
     tmp[1] = __shfl_down(tmp[1], lane_delta, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -469,13 +472,13 @@ __device__
 inline
 double __shfl_xor(MAYBE_UNDEF double var, int lane_mask, int width = warpSize) {
     static_assert(sizeof(double) == 2 * sizeof(int), "");
-    static_assert(sizeof(double) == sizeof(uint64_t), "");
+    static_assert(sizeof(double) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_xor(tmp[0], lane_mask, width);
     tmp[1] = __shfl_xor(tmp[1], lane_mask, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     double tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -485,13 +488,13 @@ long __shfl_xor(MAYBE_UNDEF long var, int lane_mask, int width = warpSize)
 {
     #ifndef _MSC_VER
     static_assert(sizeof(long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long) == sizeof(__hip_uint64_t), "");
 
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_xor(tmp[0], lane_mask, width);
     tmp[1] = __shfl_xor(tmp[1], lane_mask, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -505,13 +508,13 @@ unsigned long __shfl_xor(MAYBE_UNDEF unsigned long var, int lane_mask, int width
 {
     #ifndef _MSC_VER
     static_assert(sizeof(unsigned long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long) == sizeof(__hip_uint64_t), "");
 
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_xor(tmp[0], lane_mask, width);
     tmp[1] = __shfl_xor(tmp[1], lane_mask, width);
 
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
     #else
@@ -524,11 +527,11 @@ inline
 long long __shfl_xor(MAYBE_UNDEF long long var, int lane_mask, int width = warpSize)
 {
     static_assert(sizeof(long long) == 2 * sizeof(int), "");
-    static_assert(sizeof(long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(long long) == sizeof(__hip_uint64_t), "");
     int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_xor(tmp[0], lane_mask, width);
     tmp[1] = __shfl_xor(tmp[1], lane_mask, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
@@ -537,11 +540,11 @@ inline
 unsigned long long __shfl_xor(MAYBE_UNDEF unsigned long long var, int lane_mask, int width = warpSize)
 {
     static_assert(sizeof(unsigned long long) == 2 * sizeof(unsigned int), "");
-    static_assert(sizeof(unsigned long long) == sizeof(uint64_t), "");
+    static_assert(sizeof(unsigned long long) == sizeof(__hip_uint64_t), "");
     unsigned int tmp[2]; __builtin_memcpy(tmp, &var, sizeof(tmp));
     tmp[0] = __shfl_xor(tmp[0], lane_mask, width);
     tmp[1] = __shfl_xor(tmp[1], lane_mask, width);
-    uint64_t tmp0 = (static_cast<uint64_t>(tmp[1]) << 32ull) | static_cast<uint32_t>(tmp[0]);
+    __hip_uint64_t tmp0 = (static_cast<__hip_uint64_t>(tmp[1]) << 32ull) | static_cast<__hip_uint32_t>(tmp[0]);
     unsigned long long tmp1;  __builtin_memcpy(&tmp1, &tmp0, sizeof(tmp0));
     return tmp1;
 }
