@@ -31,6 +31,8 @@ THE SOFTWARE.
 #include "platform/program.hpp"
 #include <elf/elf.hpp>
 #include "comgrctx.hpp"
+#include "hip_comgr_helper.hpp"
+
 namespace hip {
 hipError_t ihipFree(void* ptr);
 // forward declaration of methods required for managed variables
@@ -97,6 +99,33 @@ bool CodeObject::IsClangOffloadMagicBundle(const void* data, bool& isCompressed)
   return false;
 }
 
+uint32_t CodeObject::getGenericVersion(const void* image) {
+  const Elf64_Ehdr* ehdr = reinterpret_cast<const Elf64_Ehdr*>(image);
+  return (ehdr->e_machine == EM_AMDGPU && ehdr->e_ident[EI_OSABI] == ELFOSABI_AMDGPU_HSA &&
+      ehdr->e_ident[EI_ABIVERSION] == ELFABIVERSION_AMDGPU_HSA_V6) ?
+      ((ehdr->e_flags & EF_AMDGPU_GENERIC_VERSION) >> EF_AMDGPU_GENERIC_VERSION_OFFSET) : 0;
+}
+
+bool CodeObject::isGenericTarget(const void* image) {
+  return getGenericVersion(image) >= EF_AMDGPU_GENERIC_VERSION_MIN;
+}
+
+bool CodeObject::containGenericTarget(const void *data) {
+  const auto obheader = reinterpret_cast<const __ClangOffloadBundleUncompressedHeader*>(data);
+  const auto* desc = &obheader->desc[0];
+  for (uint64_t i = 0; i < obheader->numOfCodeObjects; ++i,
+        desc = reinterpret_cast<const __ClangOffloadBundleInfo*>(
+           reinterpret_cast<uintptr_t>(&desc->bundleEntryId[0]) + desc->bundleEntryIdSize)) {
+    if (desc->size == 0) continue;
+    const void* image =
+         reinterpret_cast<const void*>(reinterpret_cast<uintptr_t>(obheader) + desc->offset);
+    if (isGenericTarget(image)) {
+      return true;
+    }
+  }
+  return false;
+}
+
 uint64_t CodeObject::ElfSize(const void* emi) { return amd::Elf::getElfSize(emi); }
 
 static bool getProcName(uint32_t EFlags, std::string& proc_name, bool& xnackSupported,
@@ -197,21 +226,16 @@ static bool getProcName(uint32_t EFlags, std::string& proc_name, bool& xnackSupp
       sramEccSupported = false;
       proc_name = "gfx90c";
       break;
-    case EF_AMDGPU_MACH_AMDGCN_GFX940:
-      xnackSupported = true;
-      sramEccSupported = true;
-      proc_name = "gfx940";
-      break;
-    case EF_AMDGPU_MACH_AMDGCN_GFX941:
-      xnackSupported = true;
-      sramEccSupported = true;
-      proc_name = "gfx941";
-      break;
     case EF_AMDGPU_MACH_AMDGCN_GFX942:
       xnackSupported = true;
       sramEccSupported = true;
       proc_name = "gfx942";
       break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX950:
+      xnackSupported = true;
+      sramEccSupported = true;
+      proc_name = "gfx950";
+      break;
     case EF_AMDGPU_MACH_AMDGCN_GFX1010:
       xnackSupported = true;
       sramEccSupported = false;
@@ -307,6 +331,36 @@ static bool getProcName(uint32_t EFlags, std::string& proc_name, bool& xnackSupp
       sramEccSupported = false;
       proc_name = "gfx1201";
       break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX9_GENERIC:
+      xnackSupported = true;
+      sramEccSupported = false;
+      proc_name = "gfx9-generic";
+      break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX9_4_GENERIC:
+      xnackSupported = true;
+      sramEccSupported = true;
+      proc_name = "gfx9-4-generic";
+      break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX10_1_GENERIC:
+      xnackSupported = true;
+      sramEccSupported = false;
+      proc_name = "gfx10-1-generic";
+      break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX10_3_GENERIC:
+      xnackSupported = false;
+      sramEccSupported = false;
+      proc_name = "gfx10-3-generic";
+      break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX11_GENERIC:
+      xnackSupported = false;
+      sramEccSupported = false;
+      proc_name = "gfx11-generic";
+      break;
+    case EF_AMDGPU_MACH_AMDGCN_GFX12_GENERIC:
+      xnackSupported = false;
+      sramEccSupported = false;
+      proc_name = "gfx12-generic";
+      break;
     default:
       return false;
   }
@@ -320,19 +374,19 @@ static bool getTripleTargetIDFromCodeObject(const void* code_object, std::string
   if (ehdr->e_ident[EI_OSABI] != ELFOSABI_AMDGPU_HSA) return false;
 
   bool isXnackSupported{false}, isSramEccSupported{false};
-
+  const char* vstr = nullptr;
   std::string proc_name;
   if (!getProcName(ehdr->e_flags, proc_name, isXnackSupported, isSramEccSupported)) return false;
   target_id = std::string(kAmdgcnTargetTriple) + '-' + proc_name;
 
   switch (ehdr->e_ident[EI_ABIVERSION]) {
     case ELFABIVERSION_AMDGPU_HSA_V2: {
-      LogPrintfInfo("[Code Object V2, target id:%s]", target_id.c_str());
+      ClPrint(amd::LOG_INFO, amd::LOG_COMGR, "[Code Object V2, target id:%s]", target_id.c_str());
       return false;
     }
 
     case ELFABIVERSION_AMDGPU_HSA_V3: {
-      LogPrintfInfo("[Code Object V3, target id:%s]", target_id.c_str());
+      ClPrint(amd::LOG_INFO, amd::LOG_COMGR, "[Code Object V3, target id:%s]", target_id.c_str());
       if (isSramEccSupported) {
         if (ehdr->e_flags & EF_AMDGPU_FEATURE_SRAMECC_V3)
           target_id += ":sramecc+";
@@ -349,11 +403,14 @@ static bool getTripleTargetIDFromCodeObject(const void* code_object, std::string
     }
 
     case ELFABIVERSION_AMDGPU_HSA_V4:
-    case ELFABIVERSION_AMDGPU_HSA_V5: {
+    case ELFABIVERSION_AMDGPU_HSA_V5:
+    case ELFABIVERSION_AMDGPU_HSA_V6: {
       if (ehdr->e_ident[EI_ABIVERSION] & ELFABIVERSION_AMDGPU_HSA_V4) {
-        LogPrintfInfo("[Code Object V4, target id:%s]", target_id.c_str());
-      } else {
-        LogPrintfInfo("[Code Object V5, target id:%s]", target_id.c_str());
+        vstr = "V4";
+      } else if (ehdr->e_ident[EI_ABIVERSION] & ELFABIVERSION_AMDGPU_HSA_V5) {
+        vstr = "V5";
+      } else if (ehdr->e_ident[EI_ABIVERSION] & ELFABIVERSION_AMDGPU_HSA_V6) {
+        vstr = "V6";
       }
       unsigned co_sram_value = (ehdr->e_flags) & EF_AMDGPU_FEATURE_SRAMECC_V4;
       if (co_sram_value == EF_AMDGPU_FEATURE_SRAMECC_OFF_V4)
@@ -364,8 +421,11 @@ static bool getTripleTargetIDFromCodeObject(const void* code_object, std::string
       unsigned co_xnack_value = (ehdr->e_flags) & EF_AMDGPU_FEATURE_XNACK_V4;
       if (co_xnack_value == EF_AMDGPU_FEATURE_XNACK_OFF_V4)
         target_id += ":xnack-";
+
       else if (co_xnack_value == EF_AMDGPU_FEATURE_XNACK_ON_V4)
         target_id += ":xnack+";
+      ClPrint(amd::LOG_INFO, amd::LOG_COMGR,
+        "[Code Object %s, target id: %s]", vstr, target_id.c_str());
       break;
     }
 
@@ -434,12 +494,11 @@ static bool getTargetIDValue(std::string& input, std::string& processor, char& s
 }
 
 static bool getTripleTargetID(std::string bundled_co_entry_id, const void* code_object,
-                              std::string& co_triple_target_id) {
+                          std::string& co_triple_target_id) {
   std::string offload_kind = trimName(bundled_co_entry_id, '-');
   if (offload_kind != kOffloadKindHipv4 && offload_kind != kOffloadKindHip &&
       offload_kind != kOffloadKindHcc)
     return false;
-
   if (offload_kind != kOffloadKindHipv4)
     return getTripleTargetIDFromCodeObject(code_object, co_triple_target_id);
 
@@ -450,7 +509,7 @@ static bool getTripleTargetID(std::string bundled_co_entry_id, const void* code_
 }
 
 static bool isCodeObjectCompatibleWithDevice(std::string co_triple_target_id,
-                                             std::string agent_triple_target_id) {
+                std::string agent_triple_target_id, unsigned int genericVersion) {
   // Primitive Check
   if (co_triple_target_id == agent_triple_target_id) return true;
 
@@ -481,14 +540,37 @@ static bool isCodeObjectCompatibleWithDevice(std::string co_triple_target_id,
   if (!agent_triple_target_id.empty()) return false;
 
   // Check for compatibility
-  if (agent_isa_processor != co_processor) return false;
+  if (genericVersion >= EF_AMDGPU_GENERIC_VERSION_MIN) {
+    // co_processor is generic target
+    if (!helpers::IsCompatibleWithGenericTarget(co_processor, agent_isa_processor))
+      return false;
+  } else if (agent_isa_processor != co_processor) {
+    return false;
+  }
   if (co_sram_ecc != ' ') {
     if (co_sram_ecc != isa_sram_ecc) return false;
   }
   if (co_xnack != ' ') {
     if (co_xnack != isa_xnack) return false;
   }
+  return true;
+}
 
+bool CodeObject::QueryGenericTarget(std::string agentTarget, std::string& processor,
+		char& sram_ecc, char& xnack) {
+  static const std::string head = std::string(kAmdgcnTargetTriple) + '-';
+  // Parse agent isa triple target id
+  if (!consume(agentTarget, head)) {
+    return false;
+  }
+  if (!getTargetIDValue(agentTarget, processor, sram_ecc, xnack)) {
+    return false;
+  }
+  if (processor.empty()) return false;
+  auto &map = helpers::GenericTargetMapping();
+  auto search = map.find(processor);
+  if (search == map.end()) return false;
+  processor = head + search->second;
   return true;
 }
 
@@ -496,7 +578,7 @@ static bool isCodeObjectCompatibleWithDevice(std::string co_triple_target_id,
 hipError_t CodeObject::ExtractCodeObjectFromFile(
     amd::Os::FileDesc fdesc, size_t fsize, const void** image,
     const std::vector<std::string>& device_names,
-    std::vector<std::pair<const void*, size_t>>& code_objs) {
+    std::vector<std::pair<const void*, size_t>>& code_objs, size_t foffset) {
   if (!amd::Os::isValidFileDesc(fdesc)) {
     return hipErrorFileNotFound;
   }
@@ -504,7 +586,7 @@ hipError_t CodeObject::ExtractCodeObjectFromFile(
   // Map the file to memory, with offset 0.
   // file will be unmapped in ModuleUnload
   // const void* image = nullptr;
-  if (!amd::Os::MemoryMapFileDesc(fdesc, fsize, 0, image)) {
+  if (!amd::Os::MemoryMapFileDesc(fdesc, fsize, foffset, image)) {
     return hipErrorInvalidValue;
   }
 
@@ -528,8 +610,9 @@ hipError_t CodeObject::ExtractCodeObjectFromMemory(
 hipError_t CodeObject::extractCodeObjectFromFatBinary(
     const void* data, const std::vector<std::string>& agent_triple_target_ids,
     std::vector<std::pair<const void*, size_t>>& code_objs) {
-  std::string magic((const char*)data, kOffloadBundleUncompressedMagicStrSize);
-  if (magic.compare(kOffloadBundleUncompressedMagicStr)) {
+  bool isCompressed = false;
+  if (!IsClangOffloadMagicBundle(data, isCompressed) || isCompressed) {
+    LogPrintfError("IsClangOffloadMagicBundle(%p) return false or isCompressed is true", data);
     return hipErrorInvalidKernelFile;
   }
 
@@ -554,13 +637,24 @@ hipError_t CodeObject::extractCodeObjectFromFatBinary(
     std::string bundleEntryId{desc->bundleEntryId, desc->bundleEntryIdSize};
 
     std::string co_triple_target_id;
+    uint32_t genericVersion = getGenericVersion(image);
     if (!getTripleTargetID(bundleEntryId, image, co_triple_target_id)) continue;
+    ClPrint(amd::LOG_INFO, amd::LOG_COMGR,
+      "bundleEntryId=%s, co_triple_target_id=%s, genericVersion=%u\n",
+      bundleEntryId.c_str(), co_triple_target_id.c_str(), genericVersion);
 
     for (size_t dev = 0; dev < agent_triple_target_ids.size(); ++dev) {
-      if (code_objs[dev].first) continue;
-      if (isCodeObjectCompatibleWithDevice(co_triple_target_id, agent_triple_target_ids[dev])) {
+      if (code_objs[dev].first != nullptr) {
+        if (!isGenericTarget(code_objs[dev].first)) {
+          continue; // Specific target already found
+        } else if(genericVersion >= EF_AMDGPU_GENERIC_VERSION_MIN) {
+          continue; // Generic target already found, no need to check another generic
+        }
+      }
+      if (isCodeObjectCompatibleWithDevice(co_triple_target_id, agent_triple_target_ids[dev],
+        genericVersion)) {
+        if (code_objs[dev].first == nullptr) --num_code_objs;
         code_objs[dev] = std::make_pair(image, image_size);
-        --num_code_objs;
       }
     }
   }
@@ -627,7 +721,7 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
   size_t num_code_objs = num_devices;
   bool isCompressed = false;
   if (!IsClangOffloadMagicBundle(data, isCompressed)) {
-    LogPrintfInfo("IsClangOffloadMagicBundle(%p) return false", data);
+    ClPrint(amd::LOG_INFO, amd::LOG_COMGR, "IsClangOffloadMagicBundle(%p) return false", data);
     // hipModuleLoadData() will possibly call here
     return hipErrorInvalidKernelFile;
   }
@@ -642,14 +736,52 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
 
 
   std::set<std::string> devicesSet{};  // To make sure device is unique
+  std::set<std::string> genericDevicesSet{}; // Used to record generic targets
+
   std::vector<const char*> bundleEntryIDs{};
   static const std::string hipv4 = kOffloadKindHipv4_;  // bundled code objects need the prefix
   for (size_t i = 0; i < num_devices; i++) {
-    devicesSet.insert(hipv4 + agent_triple_target_ids[i]);
-  }
-
-  for (auto& device : devicesSet) {
-    bundleEntryIDs.push_back(device.c_str());
+    auto res = devicesSet.insert(hipv4 + agent_triple_target_ids[i]);
+    if (res.second) {
+      // This is a new device in devicesSet
+      bundleEntryIDs.push_back(res.first->c_str());
+      std::string processor;
+      char sram_ecc = ' ', xnack = ' ';
+      if (!QueryGenericTarget(agent_triple_target_ids[i], processor, sram_ecc, xnack)) {
+        continue; // No generic target for this device
+      }
+      // Now processor is generic such as
+      // amdgcn-amd-amdhsa--gfx9-4-generic, amdgcn-amd-amdhsa--gfx11-generic
+      processor = hipv4 + processor;
+      auto ret = genericDevicesSet.insert(processor);
+      if (ret.second) {
+        // Without feature
+        bundleEntryIDs.push_back(ret.first->c_str());
+      }
+      if (xnack != ' ') {
+        ret = genericDevicesSet.insert(processor + ":xnack" + xnack);
+        if (ret.second) {
+          // Generic target with xnack feature
+          bundleEntryIDs.push_back(ret.first->c_str());
+        }
+      }
+      if (sram_ecc != ' ') {
+        processor += ":sramecc";
+        processor += sram_ecc;
+        ret = genericDevicesSet.insert(processor);
+        if (ret.second) {
+          // Generic target with sramecc feature
+          bundleEntryIDs.push_back(ret.first->c_str());
+        }
+        if (xnack != ' ') {
+          ret = genericDevicesSet.insert(processor + ":xnack" + xnack);
+          if (ret.second) {
+            // Generic target with sramecc and xnack features
+            bundleEntryIDs.push_back(ret.first->c_str());
+          }
+        }
+      }
+    }
   }
 
   do {
@@ -781,22 +913,34 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
         hipStatus = hipErrorInvalidValue;
         break;
       }
+      ClPrint(amd::LOG_DEBUG, amd::LOG_COMGR, "Found bundleEntryId=%s", bundleEntryId.c_str());
+
       // Remove bundleEntryId_
       if (!consume(bundleEntryId, kOffloadHipV4FatBinName_)) {
         // This is behavour in comgr unbundling which is subject to change.
         // So just give info.
-        LogPrintfInfo("bundleEntryId=%s isn't prefixed with %s", bundleEntryId.c_str(),
-                      kOffloadHipV4FatBinName_);
+        ClPrint(amd::LOG_INFO, amd::LOG_COMGR,
+                "bundleEntryId=%s isn't prefixed with %s", bundleEntryId.c_str(),
+                kOffloadHipV4FatBinName_);
       }
       trimNameTail(bundleEntryId, '.');  // Remove .fileExtention
 
+      // Currently we only support EF_AMDGPU_GENERIC_VERSION_MIN on generic target
+      uint32_t genericVersion =
+          bundleEntryId.find("generic") != bundleEntryId.npos ? EF_AMDGPU_GENERIC_VERSION_MIN : 0;
       char* itemData = nullptr;
       for (size_t dev = 0; dev < num_devices; ++dev) {
-        if (code_objs[dev].first) continue;
-        // LogPrintfError("agent_triple_target_ids[%zu]=%s, bundleEntryId=%s", dev,
-        //                agent_triple_target_ids[dev].c_str(), bundleEntryId.c_str());
-
-        if (bundleEntryId == agent_triple_target_ids[dev]) {
+        if (code_objs[dev].first != nullptr) {
+          if (!isGenericTarget(code_objs[dev].first)) {
+            continue;  // Specific target already found
+          } else if (genericVersion >= EF_AMDGPU_GENERIC_VERSION_MIN) {
+            continue;  // Generic target already found, no need to check another generic
+          }
+        }
+        ClPrint(amd::LOG_DEBUG, amd::LOG_COMGR, "agent_triple_target_ids[%zu]=%s, bundleEntryId=%s",
+                dev, agent_triple_target_ids[dev].c_str(), bundleEntryId.c_str());
+        if (isCodeObjectCompatibleWithDevice(bundleEntryId, agent_triple_target_ids[dev],
+                                             genericVersion)) {
           if (itemData == nullptr) {
             itemSize = 0;
             comgrStatus = amd::Comgr::get_data(item, &itemSize, nullptr);
@@ -806,17 +950,15 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
               hipStatus = hipErrorInvalidValue;
               break;
             }
-
             if (itemSize == 0) {
               // If there isn't a code object for this device,
               // amd::Comgr::do_action(AMD_COMGR_ACTION_UNBUNDLE) still returns item with
               // valid name but no data. We need continue searching for other devices
-              LogPrintfInfo(
+              ClPrint(amd::LOG_INFO, amd::LOG_COMGR,
                   "amd::Comgr::get_data() return 0 size for agent_triple_target_ids[%zu]=%s", dev,
                   agent_triple_target_ids[dev].c_str());
-              continue;
+              break;
             }
-
             // itemData should be deleted in fatbin's destructor
             itemData = new char[itemSize];
             if (itemData == nullptr) {
@@ -834,13 +976,29 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
               break;
             }
           }
+          if (code_objs[dev].first != nullptr) {
+            // This must be data of generic target
+            bool used  = false; // Still used by other devices?
+            for (size_t i = 0; i < num_devices; ++i) {
+              if (dev != i && code_objs[dev].first == code_objs[i].first) {
+                used = true;
+                break;
+              }
+            }
+            if (!used) {
+              delete[] reinterpret_cast<const char*>(code_objs[dev].first);
+            }
+          } else {
+            --num_code_objs;
+          }
           code_objs[dev] = std::make_pair(reinterpret_cast<const void*>(itemData), itemSize);
-          --num_code_objs;
-          LogPrintfInfo(
-              "Found agent_triple_target_ids[%zu]=%s: item: Data=%p(%s), "
+          ClPrint(amd::LOG_DEBUG, amd::LOG_COMGR,
+              "Found agent_triple_target_ids[%zu]=%s: item: Data=%p(%s, %s), "
               "Size=%zu, num_code_objs=%zu",
               dev, agent_triple_target_ids[dev].c_str(), itemData,
-              isCompressed ? "compressed" : "uncompressed", itemSize, num_code_objs);
+              isCompressed ? "compressed" : "uncompressed",
+              genericVersion >= EF_AMDGPU_GENERIC_VERSION_MIN ? "generic" : "non-generic",
+              itemSize, num_code_objs);
         }
       }
 
@@ -879,7 +1037,6 @@ hipError_t CodeObject::extractCodeObjectFromFatBinaryUsingComgr(
 
         std::string co_triple_target_id;
         bool valid_co = getTripleTargetID(bundleEntryId, image, co_triple_target_id);
-
         if (valid_co) {
           LogPrintfError("    %s - [Code object targetID is %s]", bundleEntryId.c_str(),
                          co_triple_target_id.c_str());
@@ -947,6 +1104,8 @@ hipError_t DynCO::loadCodeObject(const char* fname, const void* image) {
   // No Lazy loading for DynCO
   IHIP_RETURN_ONFAIL(fb_info_->BuildProgram(ihipGetDevice()));
 
+  module_ = fb_info_->Module(device_id_);
+
   // Define Global variables
   IHIP_RETURN_ONFAIL(populateDynGlobalVars());
 
@@ -965,6 +1124,19 @@ DynCO::~DynCO() {
       hipError_t err = ihipFree(elem.second->getManagedVarPtr());
       assert(err == hipSuccess);
     }
+
+    if (elem.second->getVarKind() == Var::DVK_Variable) {
+      for (auto dev : g_devices) {
+        DeviceVar* dvar = nullptr;
+        hipError_t err = elem.second->getDeviceVarPtr(&dvar, dev->deviceId());
+        assert(err == hipSuccess);
+        if (dvar != nullptr) {
+          // free also deletes the device ptr
+          err = ihipFree(dvar->device_ptr());
+          assert(err == hipSuccess);
+        }
+      }
+    }
     delete elem.second;
   }
   vars_.clear();
@@ -980,23 +1152,19 @@ DynCO::~DynCO() {
 hipError_t DynCO::getDeviceVar(DeviceVar** dvar, std::string var_name) {
   amd::ScopedLock lock(dclock_);
 
-  CheckDeviceIdMatch();
-
   auto it = vars_.find(var_name);
   if (it == vars_.end()) {
     LogPrintfError("Cannot find the Var: %s ", var_name.c_str());
     return hipErrorNotFound;
   }
 
-  hipError_t err = it->second->getDeviceVar(dvar, device_id_, module());
+  hipError_t err = it->second->getDeviceVar(dvar, device_id_, module_);
   return err;
 }
 
 hipError_t DynCO::getDynFunc(hipFunction_t* hfunc, std::string func_name) {
   amd::ScopedLock lock(dclock_);
 
-  CheckDeviceIdMatch();
-
   if (hfunc == nullptr) {
     return hipErrorInvalidValue;
   }
@@ -1008,7 +1176,13 @@ hipError_t DynCO::getDynFunc(hipFunction_t* hfunc, std::string func_name) {
   }
 
   /* See if this could be solved */
-  return it->second->getDynFunc(hfunc, module());
+  return it->second->getDynFunc(hfunc, module_);
+}
+
+bool DynCO::isValidDynFunc(const void* hfunc) {
+  amd::ScopedLock lock(dclock_);
+  return std::any_of(functions_.begin(), functions_.end(),
+                     [&](auto& it) { return it.second->isValidDynFunc(hfunc); });
 }
 
 hipError_t DynCO::initDynManagedVars(const std::string& managedVar) {
@@ -1074,7 +1248,7 @@ hipError_t DynCO::populateDynGlobalVars() {
                                      ->getDeviceProgram(*hip::getCurrentDevice()->devices()[0]);
 
   if (!dev_program->getGlobalVarFromCodeObj(&var_names)) {
-    LogPrintfError("Could not get Global vars from Code Obj for Module: 0x%x", module());
+    LogPrintfError("Could not get Global vars from Code Obj for Module: 0x%x", module_);
     return hipErrorSharedObjectSymbolNotFound;
   }
 
@@ -1102,7 +1276,7 @@ hipError_t DynCO::populateDynGlobalFuncs() {
 
   // Get all the global func names from COMGR
   if (!dev_program->getGlobalFuncFromCodeObj(&func_names)) {
-    LogPrintfError("Could not get Global Funcs from Code Obj for Module: 0x%x", module());
+    LogPrintfError("Could not get Global Funcs from Code Obj for Module: 0x%x", module_);
     return hipErrorSharedObjectSymbolNotFound;
   }
 
@@ -1138,73 +1312,93 @@ hipError_t StatCO::digestFatBinary(const void* data, FatBinaryInfo*& programs) {
   }
 
   // Create a new fat binary object and extract the fat binary for all devices.
-  programs = new FatBinaryInfo(nullptr, data);
-  IHIP_RETURN_ONFAIL(programs->ExtractFatBinary(g_devices));
-
-  return hipSuccess;
+  FatBinaryInfo* fatBinaryInfo = new FatBinaryInfo(nullptr, data);
+  hipError_t err = fatBinaryInfo->ExtractFatBinary(g_devices);
+  programs = fatBinaryInfo;
+  return err;
 }
 
-FatBinaryInfo** StatCO::addFatBinary(const void* data, bool initialized) {
+FatBinaryInfo** StatCO::addFatBinary(const void* data, bool initialized, bool& success) {
   amd::ScopedLock lock(sclock_);
+  module_to_hostModule_.insert(std::make_pair(&modules_[data], data));
 
-  if (initialized) {
-    hipError_t err = digestFatBinary(data, modules_[data]);
-    assert(err == hipSuccess);
+  if (initialized == false) {
+    success = true;
+    return &modules_[data];
   }
+
+  hipError_t err = digestFatBinary(data, modules_[data]);
+
+  success = (err == hipSuccess);
   return &modules_[data];
 }
 
 hipError_t StatCO::removeFatBinary(FatBinaryInfo** module) {
   amd::ScopedLock lock(sclock_);
 
-  auto vit = vars_.begin();
-  while (vit != vars_.end()) {
-    if (vit->second->moduleInfo() == module) {
-      delete vit->second;
-      vit = vars_.erase(vit);
-    } else {
-      ++vit;
+  auto hostVarsIter = module_to_hostVars_.find(module);
+  if (hostVarsIter != module_to_hostVars_.end()) {
+    for (auto& hostVar : hostVarsIter->second) {
+      auto varIter = vars_.find(hostVar);
+      if (varIter == vars_.end()) {
+        LogPrintfError(
+          "removeFatBinary: Unable to find module 0x%x hostVar 0x%x",
+          module, hostVar);
+      } else {
+        delete varIter->second;
+        vars_.erase(varIter);
+      }
     }
+    module_to_hostVars_.erase(hostVarsIter);
   }
 
-  auto it = managedVars_.begin();
-  while (it != managedVars_.end()) {
-    if ((*it)->moduleInfo() == module) {
+  auto managedVarsIter = managedVars_.find(module);
+  if (managedVarsIter != managedVars_.end()) {
+    for (auto& managedVar : managedVarsIter->second) {
       hipError_t err;
       for (auto dev : g_devices) {
         DeviceVar* dvar = nullptr;
-        IHIP_RETURN_ONFAIL((*it)->getStatDeviceVar(&dvar, dev->deviceId()));
-        // free also deletes the device ptr
-        err = ihipFree(dvar->device_ptr());
-        assert(err == hipSuccess);
+        IHIP_RETURN_ONFAIL(managedVar->getDeviceVarPtr(&dvar, dev->deviceId()));
+        if (dvar != nullptr) {
+          // free also deletes the device ptr
+          err = ihipFree(dvar->device_ptr());
+          assert(err == hipSuccess);
+        }
       }
-      err = ihipFree(*(static_cast<void**>((*it)->getManagedVarPtr())));
+      err = ihipFree(*(static_cast<void**>(managedVar->getManagedVarPtr())));
       assert(err == hipSuccess);
-      delete *it;
-      it = managedVars_.erase(it);
-    } else {
-      ++it;
+      delete managedVar;
     }
+    managedVars_.erase(managedVarsIter);
   }
 
-  auto fit = functions_.begin();
-  while (fit != functions_.end()) {
-    if (fit->second->moduleInfo() == module) {
-      delete fit->second;
-      fit = functions_.erase(fit);
-    } else {
-      ++fit;
+  auto hostFuncsIter = module_to_hostFunctions_.find(module);
+  if (hostFuncsIter != module_to_hostFunctions_.end()) {
+    for (auto& hostFunc : hostFuncsIter->second) {
+      auto funcIter = functions_.find(hostFunc);
+      if (funcIter == functions_.end()) {
+        LogPrintfError("removeFatBinary: Unable to find module 0x%x hostFunc 0x%x",
+                       module, hostFunc);
+      } else {
+        delete funcIter->second;
+        functions_.erase(funcIter);
+      }
     }
+    module_to_hostFunctions_.erase(hostFuncsIter);
   }
 
-  auto mit = modules_.begin();
-  while (mit != modules_.end()) {
-    if (&mit->second == module) {
-      delete mit->second;
-      mit = modules_.erase(mit);
+  auto hostModuleIter = module_to_hostModule_.find(module);
+  if (hostModuleIter != module_to_hostModule_.end()) {
+    auto hostModule = hostModuleIter->second;
+    auto moduleIter = modules_.find(hostModule);
+    if (moduleIter != modules_.end()) {
+      delete moduleIter->second;
+      modules_.erase(moduleIter);
     } else {
-      ++mit;
+      LogPrintfError("removeFatBinary: Unable to find module 0x%x via hostModule 0x%x",
+                    module, hostModule);
     }
+    module_to_hostModule_.erase(hostModuleIter);
   }
 
   return hipSuccess;
@@ -1218,6 +1412,7 @@ hipError_t StatCO::registerStatFunction(const void* hostFunction, Function* func
     delete func;
   } else {
     functions_.insert(std::make_pair(hostFunction, func));
+    module_to_hostFunctions_[func->moduleInfo()].push_back(hostFunction);
   }
 
   return hipSuccess;
@@ -1234,13 +1429,21 @@ const char* StatCO::getStatFuncName(const void* hostFunction) {
 }
 
 hipError_t StatCO::getStatFunc(hipFunction_t* hfunc, const void* hostFunction, int deviceId) {
-  amd::ScopedLock lock(sclock_);
-
   const auto it = functions_.find(hostFunction);
   if (it == functions_.end()) {
     return hipErrorInvalidSymbol;
   }
 
+  // Lazy load
+  FatBinaryInfo **module = it->second->moduleInfo();
+  if (*(module) == nullptr) {
+    amd::ScopedLock lock(sclock_);
+    if (*(module) == nullptr) {
+      hipError_t err = digestFatBinary(module_to_hostModule_[module], *module);
+      assert(err == hipSuccess);
+    }
+  }
+
   return it->second->getStatFunc(hfunc, deviceId);
 }
 
@@ -1253,6 +1456,13 @@ hipError_t StatCO::getStatFuncAttr(hipFuncAttributes* func_attr, const void* hos
     return hipErrorInvalidSymbol;
   }
 
+  // Lazy load
+  FatBinaryInfo **module = it->second->moduleInfo();
+  if (*(module) == nullptr) {
+    hipError_t err = digestFatBinary(module_to_hostModule_[module], *module);
+    assert(err == hipSuccess);
+  }
+
   return it->second->getStatFuncAttr(func_attr, deviceId);
 }
 
@@ -1265,6 +1475,7 @@ hipError_t StatCO::registerStatGlobalVar(const void* hostVar, Var* var) {
   }
 
   vars_.insert(std::make_pair(hostVar, var));
+  module_to_hostVars_[var->moduleInfo()].push_back(hostVar);
   return hipSuccess;
 }
 
@@ -1277,6 +1488,13 @@ hipError_t StatCO::getStatGlobalVar(const void* hostVar, int deviceId, hipDevice
     return hipErrorInvalidSymbol;
   }
 
+  // Lazy load
+  FatBinaryInfo **module = it->second->moduleInfo();
+  if (*(module) == nullptr) {
+    hipError_t err = digestFatBinary(module_to_hostModule_[module], *module);
+    assert(err == hipSuccess);
+  }
+
   DeviceVar* dvar = nullptr;
   IHIP_RETURN_ONFAIL(it->second->getStatDeviceVar(&dvar, deviceId));
 
@@ -1286,7 +1504,7 @@ hipError_t StatCO::getStatGlobalVar(const void* hostVar, int deviceId, hipDevice
 }
 
 hipError_t StatCO::registerStatManagedVar(Var* var) {
-  managedVars_.emplace_back(var);
+  managedVars_[var->moduleInfo()].push_back(var);
   return hipSuccess;
 }
 
@@ -1295,17 +1513,26 @@ hipError_t StatCO::initStatManagedVarDevicePtr(int deviceId) {
   hipError_t err = hipSuccess;
   if (managedVarsDevicePtrInitalized_.find(deviceId) == managedVarsDevicePtrInitalized_.end() ||
       !managedVarsDevicePtrInitalized_[deviceId]) {
-    for (auto var : managedVars_) {
-      DeviceVar* dvar = nullptr;
-      IHIP_RETURN_ONFAIL(var->getStatDeviceVar(&dvar, deviceId));
+    for (auto& vecIter : managedVars_) {
+      for (auto& var : vecIter.second) {
+        // Lazy load
+        FatBinaryInfo **module = var->moduleInfo();
+        if (*(module) == nullptr) {
+          hipError_t err = digestFatBinary(module_to_hostModule_[module], *module);
+          assert(err == hipSuccess);
+        }
 
-      hip::Stream* stream = g_devices.at(deviceId)->NullStream();
-      if (stream != nullptr) {
-        err = ihipMemcpy(reinterpret_cast<address>(dvar->device_ptr()), var->getManagedVarPtr(),
-                         dvar->size(), hipMemcpyHostToDevice, *stream);
-      } else {
-        ClPrint(amd::LOG_ERROR, amd::LOG_API, "Host Queue is NULL");
-        return hipErrorInvalidResourceHandle;
+        DeviceVar* dvar = nullptr;
+        IHIP_RETURN_ONFAIL(var->getStatDeviceVar(&dvar, deviceId));
+
+        hip::Stream* stream = g_devices.at(deviceId)->NullStream();
+        if (stream != nullptr) {
+          err = ihipMemcpy(reinterpret_cast<address>(dvar->device_ptr()), var->getManagedVarPtr(),
+                           dvar->size(), hipMemcpyHostToDevice, *stream);
+        } else {
+          ClPrint(amd::LOG_ERROR, amd::LOG_API, "Host Queue is NULL");
+          return hipErrorInvalidResourceHandle;
+        }
       }
     }
     managedVarsDevicePtrInitalized_[deviceId] = true;
