@@ -52,7 +52,7 @@ class CodeObject {
   // return code_objs{binary_ptr, binary_size}, which could be used to determine foffset
   static hipError_t ExtractCodeObjectFromFile(amd::Os::FileDesc fdesc, size_t fsize,
                     const void ** image, const std::vector<std::string>& device_names,
-                    std::vector<std::pair<const void*, size_t>>& code_objs);
+                    std::vector<std::pair<const void*, size_t>>& code_objs, size_t foffset);
 
   // Given an ptr to memory, extracts to code object for corresponding devices,
   // returns code_objs{binary_ptr, binary_size} and uniform resource indicator
@@ -65,6 +65,12 @@ class CodeObject {
 
   static bool IsClangOffloadMagicBundle(const void* data, bool& isCompressed);
 
+  static uint32_t getGenericVersion(const void* image);
+
+  static bool isGenericTarget(const void* image);
+
+  static bool containGenericTarget(const void *data);
+
   // Return size of fat bin
   static size_t getFatbinSize(const void* data, const bool isCompressed = false);
 
@@ -89,6 +95,10 @@ class CodeObject {
       const void* data, size_t size, const std::vector<std::string>& devices,
       std::vector<std::pair<const void*, size_t>>& code_objs);
 
+  // Query the generic target of agent target.
+  // Return true on successfull query, false on failure
+   static bool QueryGenericTarget(std::string agentTarget, std::string& processor,
+                char& sram_ecc, char& xnack);
  protected:
   //Given an ptr to image or file, extracts to code object
   //for corresponding devices
@@ -103,18 +113,20 @@ private:
 
 //Dynamic Code Object
 class DynCO : public CodeObject {
-  amd::Monitor dclock_{"Guards Dynamic Code object", true};
+  // Guards Dynamic Code object
+  amd::Monitor dclock_{true};
 
 public:
-  DynCO() : device_id_(ihipGetDevice()), fb_info_(nullptr) {}
+  DynCO() : device_id_(ihipGetDevice()), fb_info_(nullptr), module_(nullptr) {}
   virtual ~DynCO();
 
   //LoadsCodeObject and its data
   hipError_t loadCodeObject(const char* fname, const void* image=nullptr);
-  hipModule_t module() const { return fb_info_->Module(ihipGetDevice()); };
+  hipModule_t getModule() const { return module_; };
 
   //Gets GlobalVar/Functions from a dynamically loaded code object
   hipError_t getDynFunc(hipFunction_t* hfunc, std::string func_name);
+  bool isValidDynFunc(const void* hfunc);
   hipError_t getDeviceVar(DeviceVar** dvar, std::string var_name);
 
   hipError_t getManagedVarPointer(std::string name, void** pointer, size_t* size_ptr) const {
@@ -129,15 +141,11 @@ public:
     }
     return hipSuccess;
   }
-  // Device ID Check to check if module is launched in the same device it was loaded.
-  inline void CheckDeviceIdMatch() const {
-    guarantee(device_id_ == ihipGetDevice(), "Device mismatch from where this module is loaded,"
-              "device_id: %d ihipGetDevice:%d", device_id_, ihipGetDevice());
-  }
 
 private:
   int device_id_;
   FatBinaryInfo* fb_info_;
+  hipModule_t module_;
 
   //Maps for vars/funcs, could be keyed in with std::string name
   std::unordered_map<std::string, Function*> functions_;
@@ -151,13 +159,14 @@ private:
 
 //Static Code Object
 class StatCO: public CodeObject {
-  amd::Monitor sclock_{"Guards Static Code object", true};
+  // Guards Static Code object
+  amd::Monitor sclock_{true};
 public:
   StatCO();
   virtual ~StatCO();
 
   //Add/Remove/Digest Fat Binaries passed to us from "__hipRegisterFatBinary"
-  FatBinaryInfo** addFatBinary(const void* data, bool initialized);
+  FatBinaryInfo** addFatBinary(const void* data, bool initialized, bool& success);
   hipError_t removeFatBinary(FatBinaryInfo** module);
   hipError_t digestFatBinary(const void* data, FatBinaryInfo*& programs);
 
@@ -185,7 +194,11 @@ private:
   //Populated during __hipRegisterVars
   std::unordered_map<const void*, Var*> vars_;
   //Populated during __hipRegisterManagedVar
-  std::vector<Var*> managedVars_;
+  std::unordered_map<FatBinaryInfo**, std::vector<Var*> > managedVars_;
+  //Reverse mapping of modules to speed up removal
+  std::unordered_map<FatBinaryInfo**, const void*> module_to_hostModule_;
+  std::unordered_map<FatBinaryInfo**, std::vector<const void*> > module_to_hostFunctions_;
+  std::unordered_map<FatBinaryInfo**, std::vector<const void*> > module_to_hostVars_;
   std::unordered_map<int, bool> managedVarsDevicePtrInitalized_;
 };
 
