@@ -284,9 +284,12 @@ hipError_t hipDeviceGetAttribute(int* pi, hipDeviceAttribute_t attr, int device)
     case hipDeviceAttributePciDeviceId:
       *pi = prop.pciDeviceID;
       break;
-    case hipDeviceAttributePciDomainID:
+    case hipDeviceAttributePciDomainId:
       *pi = prop.pciDomainID;
       break;
+    case hipDeviceAttributePciChipId:
+      *pi = static_cast<int>(g_devices[device]->devices()[0]->info().pcieDeviceId_);
+      break;
     case hipDeviceAttributePersistingL2CacheMaxSize:
       *pi = prop.persistingL2CacheMaxSize;
       break;
@@ -445,6 +448,9 @@ hipError_t hipDeviceGetAttribute(int* pi, hipDeviceAttribute_t attr, int device)
     case hipDeviceAttributeAccessPolicyMaxWindowSize:
       *pi = prop.accessPolicyMaxWindowSize;
        break;
+    case hipDeviceAttributeNumberOfXccs:
+      *pi = static_cast<int>(g_devices[device]->devices()[0]->info().numberOfXccs_);
+       break;
     default:
       HIP_RETURN(hipErrorInvalidValue);
   }
@@ -519,6 +525,15 @@ hipError_t hipDeviceGetLimit(size_t* pValue, hipLimit_t limit) {
     case hipLimitStackSize:
       *pValue = hip::getCurrentDevice()->devices()[0]->StackSize();
       break;
+    case hipExtLimitScratchMin:
+      *pValue = hip::getCurrentDevice()->devices()[0]->info().scratchLimitMin;
+      break;
+    case hipExtLimitScratchMax:
+      *pValue = hip::getCurrentDevice()->devices()[0]->info().scratchLimitMax;;
+      break;
+    case hipExtLimitScratchCurrent:
+      *pValue = hip::getCurrentDevice()->devices()[0]->ScratchLimitCurrent();
+      break;
     default:
       LogPrintfError("UnsupportedLimit = %d is passed", limit);
       HIP_RETURN(hipErrorUnsupportedLimit);
@@ -574,6 +589,15 @@ hipError_t hipDeviceReset(void) {
 hipError_t hipDeviceSetCacheConfig(hipFuncCache_t cacheConfig) {
   HIP_INIT_API(hipDeviceSetCacheConfig, cacheConfig);
 
+  if (cacheConfig != hipFuncCachePreferNone && cacheConfig != hipFuncCachePreferShared &&
+      cacheConfig != hipFuncCachePreferL1 && cacheConfig != hipFuncCachePreferEqual) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+
+  if (!hip::tls.capture_streams_.empty() || !g_captureStreams.empty()) {
+    HIP_RETURN(hipErrorStreamCaptureUnsupported);
+  }
+
   // No way to set cache config yet.
 
   HIP_RETURN(hipSuccess);
@@ -596,6 +620,11 @@ hipError_t hipDeviceSetLimit(hipLimit_t limit, size_t value) {
         HIP_RETURN(hipErrorInvalidValue);
       }
       break;
+    case hipExtLimitScratchCurrent:
+      if (!hip::getCurrentDevice()->devices()[0]->UpdateScratchLimitCurrent(value)) {
+        HIP_RETURN(hipErrorInvalidValue);
+      }
+      break;
     default:
       LogPrintfError("UnsupportedLimit = %d is passed", limit);
       HIP_RETURN(hipErrorUnsupportedLimit);
@@ -609,17 +638,40 @@ hipError_t hipDeviceSetSharedMemConfig(hipSharedMemConfig config) {
       config != hipSharedMemBankSizeEightByte) {
     HIP_RETURN(hipErrorInvalidValue);
   }
+
+  if (!hip::tls.capture_streams_.empty() || !g_captureStreams.empty()) {
+    HIP_RETURN(hipErrorStreamCaptureUnsupported);
+  }
+
   // No way to set cache config yet.
 
   HIP_RETURN(hipSuccess);
 }
 
+hipError_t hipDeviceGetTexture1DLinearMaxWidth(size_t* maxWidthInElements,
+ const hipChannelFormatDesc* fmtDesc, int device) {
+  HIP_INIT_API(hipDeviceGetTexture1DLinearMaxWidth, maxWidthInElements, fmtDesc, device);
+  if (maxWidthInElements == nullptr || fmtDesc == nullptr) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+  hipDeviceProp_tR0600 prop = {0};
+  HIP_RETURN_ONFAIL(ihipGetDeviceProperties(&prop, device));
+  // Calculate element size according to fmtDesc
+  size_t elementSize = (fmtDesc->x + fmtDesc->y
+  + fmtDesc->z + fmtDesc->w) / 8; // Convert from bits to bytes
+  if (elementSize == 0) {
+    HIP_RETURN(hipErrorInvalidValue);
+  }
+  *maxWidthInElements = prop.maxTexture1DLinear / elementSize;
+  HIP_RETURN(hipSuccess);
+}
+
 hipError_t hipDeviceSynchronize() {
   HIP_INIT_API(hipDeviceSynchronize);
   CHECK_SUPPORTED_DURING_CAPTURE();
   constexpr bool kDoWaitForCpu = false;
   hip::getCurrentDevice()->SyncAllStreams(kDoWaitForCpu);
-  HIP_RETURN(hipSuccess);
+  HIP_RETURN_DURATION(hipSuccess);
 }
 
 int ihipGetDevice() {
@@ -633,16 +685,17 @@ int ihipGetDevice() {
 hipError_t hipGetDevice(int* deviceId) {
   HIP_INIT_API(hipGetDevice, deviceId);
 
-  if (deviceId != nullptr) {
-    int dev = ihipGetDevice();
-    if (dev == -1) {
-      HIP_RETURN(hipErrorNoDevice);
-    }
-    *deviceId = dev;
-    HIP_RETURN(hipSuccess);
-  } else {
+  if (deviceId == nullptr) {
     HIP_RETURN(hipErrorInvalidValue);
   }
+
+  Device* device = hip::getCurrentDevice();
+  if (device == nullptr) {
+    HIP_RETURN(hipErrorNoDevice);
+  }
+
+  *deviceId = device->deviceId();
+  HIP_RETURN(hipSuccess, *deviceId);
 }
 
 hipError_t hipGetDeviceCount(int* count) {
@@ -662,6 +715,12 @@ hipError_t hipGetDeviceFlags(unsigned int* flags) {
 
 hipError_t hipSetDevice(int device) {
   HIP_INIT_API_NO_RETURN(hipSetDevice, device);
+
+  // Check if the device is already set
+  if (hip::tls.device_ != nullptr && hip::tls.device_->deviceId() == device) {
+    HIP_RETURN(hipSuccess);
+  }
+
   if (static_cast<unsigned int>(device) < g_devices.size()) {
     hip::setCurrentDevice(device);
 
