@@ -20,6 +20,7 @@
 
 #include <hip/hip_runtime.h>
 #include "hip_internal.hpp"
+#include "hip_graph_internal.hpp"
 #include "hip_event.hpp"
 #include "thread/monitor.hpp"
 #include "hip_prof_api.h"
@@ -72,9 +73,10 @@ bool Stream::Create() {
 }
 
 // ================================================================================================
-void Stream::Destroy(hip::Stream* stream) {
+void Stream::Destroy(hip::Stream* stream, bool forceDestroy) {
   stream->device().removeFromActiveQueues(stream);
   stream->device_->RemoveStream(stream);
+  stream->SetForceDestroy(forceDestroy);
   stream->release();
   stream = nullptr;
 }
@@ -322,9 +324,7 @@ hipError_t hipDeviceGetStreamPriorityRange(int* leastPriority, int* greatestPrio
 // ================================================================================================
 hipError_t hipStreamGetFlags_common(hipStream_t stream, unsigned int* flags) {
   if ((flags != nullptr) && (stream != nullptr)) {
-    if (!hip::isValid(stream)) {
-      return hipErrorContextIsDestroyed;
-    }
+    getStreamPerThread(stream);
     *flags = reinterpret_cast<hip::Stream*>(stream)->Flags();
   } else {
     return hipErrorInvalidValue;
@@ -348,9 +348,7 @@ hipError_t hipStreamGetFlags_spt(hipStream_t stream, unsigned int* flags) {
 
 // ================================================================================================
 hipError_t hipStreamSynchronize_common(hipStream_t stream) {
-  if (!hip::isValid(stream)) {
-    HIP_RETURN(hipErrorContextIsDestroyed);
-  }
+  getStreamPerThread(stream);
   if (stream != nullptr && stream != hipStreamLegacy) {
     // If still capturing return error
     if (hip::Stream::StreamCaptureOngoing(stream) == true) {
@@ -361,10 +359,7 @@ hipError_t hipStreamSynchronize_common(hipStream_t stream) {
   if (stream == nullptr) {
     // Do cpu wait on null stream and only on blocking streams
     constexpr bool WaitblockingStreamOnly = true;
-    getCurrentDevice()->SyncAllStreams(true, WaitblockingStreamOnly);
-
-    // Release freed memory for all memory pools on the device
-    getCurrentDevice()->ReleaseFreedMemory();
+    getCurrentDevice()->SyncAllStreams(false, WaitblockingStreamOnly);
   } else {
     constexpr bool wait = false;
     auto hip_stream = hip::getStream(stream, wait);
@@ -397,12 +392,9 @@ hipError_t hipStreamDestroy(hipStream_t stream) {
   if (stream == nullptr) {
     HIP_RETURN(hipErrorInvalidHandle);
   }
-  if (stream == hipStreamPerThread) {
+  if (stream == hipStreamPerThread || stream == hipStreamLegacy) {
     HIP_RETURN(hipErrorInvalidResourceHandle);
   }
-  if (!hip::isValid(stream)) {
-    HIP_RETURN(hipErrorContextIsDestroyed);
-  }
   hip::Stream* s = reinterpret_cast<hip::Stream*>(stream);
   if (s->GetCaptureStatus() != hipStreamCaptureStatusNone) {
     if (s->GetParentStream() != nullptr) {
@@ -449,11 +441,15 @@ void WaitThenDecrementSignal(hipStream_t stream, hipError_t status, void* user_d
 
 // ================================================================================================
 hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsigned int flags) {
+  if (flags != hipEventWaitDefault && flags != hipEventWaitExternal) {
+    return hipErrorInvalidValue;
+  }
   hipError_t status = hipSuccess;
-  if (event == nullptr || !hip::isValid(stream)) {
+  if (event == nullptr) {
     return hipErrorInvalidHandle;
   }
-  hip::Stream* waitStream = reinterpret_cast<hip::Stream*>(stream);
+  getStreamPerThread(stream);
+  hip::Stream* waitStream = hip::getStream(stream);
   hip::Event* e = reinterpret_cast<hip::Event*>(event);
   auto eventStreamHandle = reinterpret_cast<hipStream_t>(e->GetCaptureStream());
   // the stream associated with the device might have been destroyed
@@ -464,7 +460,16 @@ hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsig
   }
 
   hip::Stream* eventStream = reinterpret_cast<hip::Stream*>(eventStreamHandle);
-  if (eventStream != nullptr && eventStream->IsEventCaptured(event) == true) {
+  if (flags == hipEventWaitExternal) {
+    auto lastCapturedNodes = waitStream->GetLastCapturedNodes();
+    hip::GraphNode* pGraphNode = waitStream->GetCaptureGraph()->AddExternalEventWaitNode(
+                                      reinterpret_cast<hip::GraphNode*>(lastCapturedNodes.data()),
+                                      lastCapturedNodes.size(),
+                                      event);
+    waitStream->SetLastCapturedNode(pGraphNode);
+    return hipSuccess;
+  }
+  else if (eventStream != nullptr && eventStream->IsEventCaptured(event) == true) {
     ClPrint(amd::LOG_INFO, amd::LOG_API,
           "[hipGraph] Current capture node StreamWaitEvent on stream : %p, Event %p", stream,
           event);
@@ -480,19 +485,17 @@ hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsig
     }
     waitStream->AddCrossCapturedNode(e->GetNodesPrevToRecorded());
   } else {
-    if (flags != 0) {
-      return hipErrorInvalidValue;
-    }
     if (eventStream != nullptr) {
       if (eventStream->GetCaptureStatus() == hipStreamCaptureStatusActive) {
         // If stream is capturing but event is not recorded on event's stream.
         return hipErrorStreamCaptureIsolation;
       }
-      if ((waitStream != nullptr) && (eventStream->DeviceId() == waitStream->DeviceId())) {
+      if ((waitStream != nullptr && stream != hipStreamLegacy) &&
+          (eventStream->DeviceId() == waitStream->DeviceId())) {
         eventStream->GetDevice()->AddSafeStream(eventStream, waitStream);
       }
     }
-    status = e->streamWait(stream, flags);
+    status = e->streamWait(waitStream, flags);
   }
   return status;
 }
@@ -512,9 +515,7 @@ hipError_t hipStreamWaitEvent_spt(hipStream_t stream, hipEvent_t event, unsigned
 
 // ================================================================================================
 hipError_t hipStreamQuery_common(hipStream_t stream) {
-  if (!hip::isValid(stream)) {
-    return hipErrorContextIsDestroyed;
-  }
+  getStreamPerThread(stream);
   if (stream != nullptr) {
     // If still capturing return error
     if (hip::Stream::StreamCaptureOngoing(stream) == true) {
@@ -567,9 +568,7 @@ hipError_t hipStreamQuery_spt(hipStream_t stream) {
 }
 
 hipError_t streamCallback_common(hipStream_t stream, StreamCallback* cbo, void* userData) {
-  if (!hip::isValid(stream)) {
-    return hipErrorContextIsDestroyed;
-  }
+  getStreamPerThread(stream);
 
   hip::Stream* hip_stream = hip::getStream(stream);
   amd::Command* last_command = hip_stream->getLastQueuedCommand(true);
@@ -619,6 +618,19 @@ hipError_t hipStreamAddCallback_common(hipStream_t stream, hipStreamCallback_t c
   if (callback == nullptr || flags != 0) {
     return hipErrorInvalidValue;
   }
+
+  if (stream != nullptr && stream != hipStreamLegacy && hip::isValid(stream)) {
+    hip::Stream* s = reinterpret_cast<hip::Stream*>(stream);
+    if (s->GetCaptureStatus() != hipStreamCaptureStatusNone) {
+      s->SetCaptureStatus(hipStreamCaptureStatusInvalidated);
+      return hipErrorStreamCaptureUnsupported;
+    }
+  } else if (Stream::StreamCaptureBlocking() == true) {
+    // If any of the blocking streams is capturing, return error for implicit capture and
+    // invalidate capture for all capturing streams
+    CHECK_STREAM_CAPTURING();
+  }
+
   StreamCallback* cbo = new StreamAddCallback(stream, callback, userData);
   return streamCallback_common(stream, cbo, userData);
 }
@@ -689,9 +701,7 @@ hipError_t hipStreamGetPriority_common(hipStream_t stream, int* priority) {
   }
 
   if ((priority != nullptr) && (stream != nullptr)) {
-    if (!hip::isValid(stream)) {
-      return hipErrorContextIsDestroyed;
-    }
+    getStreamPerThread(stream);
     *priority = static_cast<int>(reinterpret_cast<hip::Stream*>(stream)->GetPriority());
   } else {
     return hipErrorInvalidValue;
