@@ -85,13 +85,13 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
 
   // Validate input params
   if (pTexObject == nullptr || pResDesc == nullptr || pTexDesc == nullptr) {
-    return hipErrorInvalidValue;
+    return hipErrorInvalidChannelDescriptor;
   }
 
   // pResViewDesc can only be specified if the type of resource is a HIP array or a HIP mipmapped array.
   if ((pResViewDesc != nullptr) &&
       ((pResDesc->resType != hipResourceTypeArray) && (pResDesc->resType != hipResourceTypeMipmappedArray))) {
-    return hipErrorInvalidValue;
+    return hipErrorUnknown;
   }
 
   // If hipResourceDesc::resType is set to hipResourceTypeArray,
@@ -124,7 +124,7 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
       return hipErrorNotSupported;
     }
     if (pResDesc->res.mipmap.mipmap == nullptr || pTexDesc->normalizedCoords == 0) {
-      return hipErrorInvalidValue;
+      return hipErrorInvalidChannelDescriptor;
     }
   }
 
@@ -135,7 +135,7 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
       ((pResDesc->res.linear.devPtr == nullptr) ||
        (!amd::isMultipleOf(pResDesc->res.linear.devPtr, info.imageBaseAddressAlignment_)) ||
        (pResDesc->res.linear.sizeInBytes >= info.imageMaxBufferSize_ * hip::getElementSize(pResDesc->res.linear.desc)))) {
-    return hipErrorInvalidValue;
+    return hipErrorInvalidChannelDescriptor;
   }
 
   // If hipResourceDesc::resType is set to hipResourceTypePitch2D,
@@ -147,8 +147,8 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
   if ((pResDesc->resType == hipResourceTypePitch2D) &&
       ((pResDesc->res.pitch2D.devPtr == nullptr) ||
        (!amd::isMultipleOf(pResDesc->res.pitch2D.devPtr, info.imageBaseAddressAlignment_)) ||
-       (pResDesc->res.pitch2D.width >= info.image2DMaxWidth_) ||
-       (pResDesc->res.pitch2D.height >= info.image2DMaxHeight_) ||
+       (pResDesc->res.pitch2D.width > info.image2DMaxWidth_) ||
+       (pResDesc->res.pitch2D.height > info.image2DMaxHeight_) ||
        (!amd::isMultipleOf(pResDesc->res.pitch2D.pitchInBytes, info.imagePitchAlignment_)))) {
     // TODO check pitch limits.
     return hipErrorInvalidValue;
@@ -173,17 +173,20 @@ hipError_t ihipCreateTextureObject(hipTextureObject_t* pTexObject,
   }
 
   // TODO ROCclr assumes all dimensions have the same addressing mode.
-  cl_addressing_mode addressMode = CL_ADDRESS_NONE;
+  cl_addressing_mode addressMode[3] = { CL_ADDRESS_NONE, CL_ADDRESS_NONE, CL_ADDRESS_NONE};
   // If hipTextureDesc::normalizedCoords is set to zero,
   // hipAddressModeWrap and hipAddressModeMirror won't be supported
   // and will be switched to hipAddressModeClamp.
-  if ((pTexDesc->normalizedCoords == 0) &&
-      ((pTexDesc->addressMode[0] == hipAddressModeWrap) || (pTexDesc->addressMode[0] == hipAddressModeMirror))) {
-    addressMode = hip::getCLAddressingMode(hipAddressModeClamp);
-  }
-  // hipTextureDesc::addressMode is ignored if hipResourceDesc::resType is hipResourceTypeLinear
-  else if (pResDesc->resType != hipResourceTypeLinear) {
-    addressMode = hip::getCLAddressingMode(pTexDesc->addressMode[0]);
+  for (int i = 0; i < 3; i++) {
+    if ((pTexDesc->normalizedCoords == 0) &&
+        ((pTexDesc->addressMode[i] == hipAddressModeWrap) ||
+            (pTexDesc->addressMode[i] == hipAddressModeMirror))) {
+      addressMode[i] = hip::getCLAddressingMode(hipAddressModeClamp);
+    }
+    // hipTextureDesc::addressMode is ignored if hipResourceDesc::resType is hipResourceTypeLinear
+    else if (pResDesc->resType != hipResourceTypeLinear) {
+      addressMode[i] = hip::getCLAddressingMode(pTexDesc->addressMode[i]);
+    }
   }
 
 #ifndef CL_FILTER_NONE
@@ -392,7 +395,9 @@ hipError_t ihipDestroyTextureObject(hipTextureObject_t texObject) {
     return hipErrorNotSupported;
   }
 
-  texObject->image->release();
+  if (texObject != nullptr && texObject->image) {
+    texObject->image->release();
+  }
 
   // The texture object always owns the sampler SRD.
   texObject->sampler->release();
@@ -529,9 +534,15 @@ hipError_t ihipBindTexture(size_t* offset,
                            const void* devPtr,
                            const hipChannelFormatDesc* desc,
                            size_t size) {
-  if ((texref == nullptr) ||
-      (devPtr == nullptr) ||
-      (desc == nullptr)) {
+  if (texref == nullptr) {
+    return hipErrorUnknown;
+  }
+
+  if (devPtr == nullptr) {
+    return hipErrorNotFound;
+  }
+
+  if (desc == nullptr) {
     return hipErrorInvalidValue;
   }
 
@@ -554,7 +565,8 @@ hipError_t ihipBindTexture(size_t* offset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.linear.devPtr = static_cast<char*>(const_cast<void*>(devPtr)) - *offset;
+  resDesc.res.linear.devPtr =
+      static_cast<char*>(const_cast<void*>(devPtr)) - (offset != nullptr ? *offset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texref);
 
   return ihipCreateTextureObject(const_cast<hipTextureObject_t*>(&texref->textureObject), &resDesc, &texDesc, nullptr);
@@ -569,7 +581,8 @@ hipError_t ihipBindTexture2D(size_t* offset,
                              size_t pitch) {
   if ((texref == nullptr) ||
       (devPtr == nullptr) ||
-      (desc == nullptr)) {
+      (desc == nullptr) ||
+      (pitch == 0)) {
     return hipErrorInvalidValue;
   }
 
@@ -594,7 +607,8 @@ hipError_t ihipBindTexture2D(size_t* offset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.pitch2D.devPtr = static_cast<char*>(const_cast<void*>(devPtr)) - *offset;
+  resDesc.res.pitch2D.devPtr =
+      static_cast<char*>(const_cast<void*>(devPtr)) - (offset != nullptr ? *offset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texref);
 
   return ihipCreateTextureObject(const_cast<hipTextureObject_t*>(&texref->textureObject), &resDesc, &texDesc, nullptr);
@@ -609,6 +623,13 @@ hipError_t hipBindTexture2D(size_t* offset,
                             size_t pitch) {
   HIP_INIT_API(hipBindTexture2D, offset, texref, devPtr, desc, width, height, pitch);
 
+  if (texref == nullptr) {
+    HIP_RETURN(hipErrorUnknown);
+  }
+  if (devPtr == nullptr) {
+    HIP_RETURN(hipErrorNotFound);
+  }
+
   hipDeviceptr_t refDevPtr = nullptr;
   size_t refDevSize = 0;
 
@@ -628,12 +649,6 @@ hipError_t hipBindTexture2D(size_t* offset,
 hipError_t ihipBindTextureToArray(const textureReference* texref,
                                   hipArray_const_t array,
                                   const hipChannelFormatDesc* desc) {
-  if ((texref == nullptr) ||
-      (array == nullptr) ||
-      (desc == nullptr)) {
-    return hipErrorInvalidValue;
-  }
-
   // Any previous address or HIP array state associated with the texture reference is superseded by this function.
   // Any memory previously bound to hTexRef is unbound.
   // No need to check for errors.
@@ -659,6 +674,10 @@ hipError_t hipBindTextureToArray(const textureReference* texref,
                                  const hipChannelFormatDesc* desc) {
   HIP_INIT_API(hipBindTextureToArray, texref, array, desc);
 
+  if ((texref == nullptr) || (array == nullptr) || (desc == nullptr)) {
+    return hipErrorInvalidValue;
+  }
+
   hipDeviceptr_t refDevPtr = nullptr;
   size_t refDevSize = 0;
   HIP_RETURN_ONFAIL(PlatformState::instance().getStatGlobalVar(texref, ihipGetDevice(), &refDevPtr,
@@ -779,7 +798,11 @@ hipError_t hipGetTextureAlignmentOffset(size_t* offset,
                                         const textureReference* texref) {
   HIP_INIT_API(hipGetTextureAlignmentOffset, offset, texref);
 
-  if ((offset == nullptr) || (texref == nullptr)) {
+  if (texref == nullptr) {
+    HIP_RETURN(hipErrorInvalidTexture);
+  }
+
+  if (offset == nullptr) {
     HIP_RETURN(hipErrorInvalidValue);
   }
   amd::Device* device = hip::getCurrentDevice()->devices()[0];
@@ -1001,6 +1024,11 @@ hipError_t hipTexRefSetArray(textureReference* texRef,
     HIP_RETURN(err);
   }
 
+  // If flag = HIP_TRSA_OVERRIDE_FORMAT,
+  // Override the texref format with a format inferred from the array
+  texRef->format = array->Format;
+  texRef->numChannels = array->NumChannels;
+
   hipResourceDesc resDesc = {};
   resDesc.resType = hipResourceTypeArray;
   resDesc.res.array.array = const_cast<hipArray_t>(array);
@@ -1090,7 +1118,7 @@ hipError_t hipTexRefSetAddress(size_t* ByteOffset,
   }
 
   // Align the user ptr to HW requirments.
-  resDesc.res.linear.devPtr = static_cast<char*>(dptr) - *ByteOffset;
+  resDesc.res.linear.devPtr = static_cast<char*>(dptr) - (ByteOffset ? *ByteOffset : 0);
   hipTextureDesc texDesc = hip::getTextureDesc(texRef);
 
   err = ihipCreateTextureObject(&texRef->textureObject, &resDesc, &texDesc, nullptr);
@@ -1262,7 +1290,7 @@ hipError_t hipTexRefGetMaxAnisotropy(int* pmaxAnsio,
 
   *pmaxAnsio = texRef->maxAnisotropy;
 
-  HIP_RETURN(hipErrorInvalidValue);
+  HIP_RETURN(hipSuccess);
 }
 
 hipError_t hipTexRefGetMipmapFilterMode(hipTextureFilterMode* pfm,
@@ -1513,12 +1541,8 @@ hipError_t hipTexObjectCreate(hipTextureObject_t* pTexObject,
                               const HIP_RESOURCE_VIEW_DESC* pResViewDesc) {
   HIP_INIT_API(hipTexObjectCreate, pTexObject, pResDesc, pTexDesc, pResViewDesc);
 
-  if (pTexObject == nullptr) {
-    HIP_RETURN(hipErrorNotInitialized);
-  }
-
-  if (pResDesc == nullptr || pTexDesc == nullptr) {
-    HIP_RETURN(hipErrorInvalidContext);
+  if ((pTexObject == nullptr) || (pResDesc == nullptr) || (pTexDesc == nullptr)) {
+    HIP_RETURN(hipErrorInvalidValue);
   }
 
   hipResourceDesc resDesc = hip::getResourceDesc(*pResDesc);
