@@ -43,14 +43,18 @@ const char* BlitLinearSourceCode = BLIT_KERNELS(
 
   extern void __amd_streamOpsWait(__global uint*, __global ulong*, ulong, ulong, ulong);
 
+  extern void __amd_batchMemOp(__global void*, uint count);
+
   extern void __ockl_dm_init_v1(ulong, ulong, uint, uint);
 
   extern void __ockl_gws_init(uint nwm1, uint rid);
 
   __kernel void __amd_rocclr_fillBufferAligned(
     __global void* buf, __constant uchar* pattern,
-    uint pattern_size, uint alignment, ulong end_ptr, uint next_chunk) {
-    int id = get_global_id(0);
+    uint pattern_size, uint alignment, ulong end_ptr, uint next_chunk, uint workgroup_size) {
+    uint l = __builtin_amdgcn_workitem_id_x();
+    uint g = __builtin_amdgcn_workgroup_id_x();
+    ulong id = (g * workgroup_size + l);
     long cur_id = id * pattern_size;
     if (alignment == sizeof(ulong2)) {
       __global ulong2* bufULong2 = (__global ulong2*)buf;
@@ -120,9 +124,12 @@ const char* BlitLinearSourceCode = BLIT_KERNELS(
   }
 
   __kernel void __amd_rocclr_copyBuffer(__global uchar* src, __global uchar* dst,
-                                          ulong size, uint remainder,
-                                          uint aligned_size, ulong end_ptr, uint next_chunk) {
-    ulong id = get_global_id(0);
+                                        ulong size, uint remainder,
+                                        uint aligned_size, ulong end_ptr,
+                                        uint next_chunk, uint workgroup_size) {
+    uint l = __builtin_amdgcn_workitem_id_x();
+    uint g = __builtin_amdgcn_workgroup_id_x();
+    ulong id = (g * workgroup_size + l);
     ulong id_remainder = id;
 
     if (aligned_size == sizeof(ulong2)) {
@@ -162,6 +169,10 @@ const char* BlitLinearSourceCode = BLIT_KERNELS(
                                                    ulong4 srcRect, ulong4 dstRect, ulong4 size) {
     __amd_copyBufferRectAligned(src, dst, srcRect, dstRect, size);
   }
+
+  __kernel void __amd_rocclr_batchMemOp(__global void* params, uint count) {
+    __amd_batchMemOp(params, count);
+  }
 );
 
 const char* HipExtraSourceCode = BLIT_KERNELS(
@@ -254,7 +265,6 @@ const char* BlitImageSourceCode = BLIT_KERNELS(
     __amd_copyImageToBuffer(src, dstUInt, dstUShort, dstUChar, srcOrigin, dstOrigin, size, format,
                             pitch);
   }
-
 );
 
 }  // namespace amd::device
