@@ -92,6 +92,7 @@ class SvmMapMemoryCommand;
 class SvmUnmapMemoryCommand;
 class SvmPrefetchAsyncCommand;
 class StreamOperationCommand;
+class BatchMemoryOperationCommand;
 class VirtualMapCommand;
 class ExternalSemaphoreCmd;
 class Isa;
@@ -125,6 +126,16 @@ enum MemRangeAttribute : uint32_t {
 constexpr int CpuDeviceId = static_cast<int>(-1);
 constexpr int InvalidDeviceId = static_cast<int>(-2);
 
+// Max scratch size is device dependent.
+constexpr size_t kWave32 = 32;
+constexpr size_t kWave64 = 64;
+constexpr size_t kScratchBits12X = 18;
+constexpr size_t kScratchBits9X = 15;
+constexpr size_t kCompilerRequired = 64;
+constexpr size_t kMaxStackSize12X = (((1 << kScratchBits12X) - 1) * 256 / kWave32) - kCompilerRequired;
+constexpr size_t kMaxStackSize11X = (((1 << kScratchBits9X) - 1) * 256 / kWave32) - kCompilerRequired;
+constexpr size_t kMaxStackSize9X = (((1 << kScratchBits9X) - 1) * 256 / kWave64) - kCompilerRequired;
+
 enum class ExternalSemaphoreHandleType : uint32_t {
   OpaqueFd = 1,        // Handle is an opaque file descriptor
   OpaqueWin32 = 2,     // Handle is an opaque shared NT handle
@@ -647,6 +658,11 @@ struct Info : public amd::EmbeddedObject {
   uint32_t luidLowPart_;        //!< Luid low 4 bytes, available in Windows only
   uint32_t luidHighPart_;       //!< Luid high 4 bytes, available in Windows only
   uint32_t luidDeviceNodeMask_; //!< Luid node mask
+
+  size_t scratchLimitMin; //! Minimum size of scratch limit of this device memory in bytes.
+  size_t scratchLimitMax; //! Maximum size of scratch limit of this device memory in bytes.
+
+  uint32_t numberOfXccs_; //! The number of XCC(s) on the device
 };
 
 //! Device settings
@@ -672,8 +688,6 @@ class Settings : public amd::HeapObject {
       uint customHostAllocator_ : 1;  //!< True if device has custom host allocator
                                       //  that replaces generic OS allocation routines
       uint supportDepthsRGB_ : 1;     //!< Support DEPTH and sRGB channel order format
-      uint reportFMAF_ : 1;           //!< Report FP_FAST_FMAF define in CL program
-      uint reportFMA_ : 1;            //!< Report FP_FAST_FMA define in CL program
       uint singleFpDenorm_ : 1;       //!< Support Single FP Denorm
       uint hsailExplicitXnack_ : 1;   //!< Xnack in hsail path for this device
       uint useLightning_ : 1;         //!< Enable LC path for this device
@@ -688,7 +702,7 @@ class Settings : public amd::HeapObject {
       uint gwsInitSupported_:1;       //!< Check if GWS is supported on this machine.
       uint kernel_arg_opt_: 1;        //!< Enables kernel arg optimization for blit kernels
       uint kernel_arg_impl_ : 2;      //!< Kernel argument implementation
-      uint reserved_ : 7;
+      uint reserved_ : 12;
     };
     uint value_;
   };
@@ -708,6 +722,8 @@ class Settings : public amd::HeapObject {
   //! Enable the specified extension
   void enableExtension(uint name) { extensions_ |= static_cast<uint64_t>(1) << name; }
 
+  size_t stagedXferSize_ = 0;     //!< Staged buffer size
+
  private:
   //! Disable copy constructor
   Settings(const Settings&);
@@ -938,6 +954,9 @@ class Memory : public amd::HeapObject {
   //! Get current access of the memory in device.
   MemAccess GetAccess() const { return memAccess_; }
 
+  //! Retrieves shareable handle for hipMalloc'ed address range.
+  virtual bool GetFDHandleForMem(void* dev_ptr, size_t size, bool vmm, void* handle) { return false; }
+
  protected:
   enum Flags {
     HostMemoryDirectAccess = 0x00000001,  //!< GPU has direct access to the host memory
@@ -962,7 +981,7 @@ class Memory : public amd::HeapObject {
   //! not a physical map. When a memory object does not use USE_HOST_PTR we
   //! can use a remote resource and DMA, avoiding the additional CPU memcpy.
   amd::Memory* mapMemory_;            //!< Memory used as map target buffer
-  volatile size_t indirectMapCount_;  //!< Number of maps
+  std::atomic<size_t> indirectMapCount_;  //!< Number of maps
   std::unordered_map<const void*, WriteMapInfo>
       writeMapInfo_;  //!< Saved write map info for partial unmap
 
@@ -1266,7 +1285,7 @@ class VirtualDevice : public amd::HeapObject {
   VirtualDevice(amd::Device& device)
     : device_(device)
     , blitMgr_(NULL)
-    , execution_("Virtual device execution lock", true)
+    , execution_(true) /* Virtual device execution lock */
     , index_(0) {}
 
   //! Destroy this virtual device.
@@ -1274,7 +1293,7 @@ class VirtualDevice : public amd::HeapObject {
 
   //! Return the physical device for this virtual device.
   const amd::Device& device() const { return device_(); }
-
+  virtual uint64_t getQueueID() = 0;
   virtual void submitReadMemory(amd::ReadMemoryCommand& cmd) = 0;
   virtual void submitWriteMemory(amd::WriteMemoryCommand& cmd) = 0;
   virtual void submitCopyMemory(amd::CopyMemoryCommand& cmd) = 0;
@@ -1306,9 +1325,14 @@ class VirtualDevice : public amd::HeapObject {
     ShouldNotReachHere();
   }
   virtual void submitStreamOperation(amd::StreamOperationCommand& cmd) { ShouldNotReachHere(); }
+  virtual void submitBatchMemoryOperation(amd::BatchMemoryOperationCommand& cmd) {
+    ShouldNotReachHere();
+  }
   virtual void submitVirtualMap(amd::VirtualMapCommand& cmd) { ShouldNotReachHere(); }
 
   virtual address allocKernelArguments(size_t size, size_t alignment) { return nullptr; }
+  virtual void ReleaseAllHwQueues() {}
+  virtual void ReleaseHwQueue() {}
 
   //! Get the blit manager object
   device::BlitManager& blitMgr() const { return *blitMgr_; }
@@ -1355,10 +1379,25 @@ class VirtualDevice : public amd::HeapObject {
 }  // namespace amd::device
 
 namespace amd {
-
+/*! IHIP IPC MEMORY Structure */
+#define AMD_IPC_MEM_HANDLE_SIZE 32
 //! MemoryObject map lookup  class
 class MemObjMap : public AllStatic {
  public:
+  struct IpcMemHandle {
+    char ipc_handle[AMD_IPC_MEM_HANDLE_SIZE];  ///< ipc memory handle on ROCr
+    size_t psize;                        ///< Total size of the device memory allocation
+    size_t poffset;                      ///< Offset within the allocation
+    int owners_process_id;               ///< ID of the process that owns the allocation
+    char reserved[LP64_SWITCH(20, 12)];  ///< Reserved for future extensions
+
+    bool operator<(const IpcMemHandle& h) const {
+      int cmp = std::memcmp(ipc_handle, h.ipc_handle, AMD_IPC_MEM_HANDLE_SIZE);
+      if (cmp != 0) return cmp < 0;
+
+      return poffset < h.poffset;
+    }
+  };
   //!< add the host mem pointer and buffer in the container
   static void AddMemObj(const void* k, amd::Memory* v);
 
@@ -1378,6 +1417,13 @@ class MemObjMap : public AllStatic {
   //!< Same as FindMemObj but for virtual addressing
   static amd::Memory* FindVirtualMemObj(const void* k);
 
+  //!< Same as AddMemObj but for virtual ipc handle to MemObj mapping
+  static void AddIpcHandleMemObj(const IpcMemHandle& k, amd::Memory* v);
+  //!< Remove entry from the map by searching values
+  static void RemoveIpcHandleMemObj(amd::Memory* v);
+  //!< Same as FindMemObj but for ipc handle to MemObj mapping
+  static amd::Memory* FindIpcHandleMemObj(const IpcMemHandle& k);
+
  private:
   //!< the mem object<->hostptr information container
   static std::map<uintptr_t, amd::Memory*> MemObjMap_;
@@ -1385,6 +1431,8 @@ class MemObjMap : public AllStatic {
   static std::map<uintptr_t, amd::Memory*> VirtualMemObjMap_;
   //!< Shared read/write lock
   static std::shared_mutex AllocatedLock_;
+  //!< the ipc handle<->mem object information container
+  static std::map<IpcMemHandle, amd::Memory*> IpcHandleMemObjMap_;
 };
 
 /// @brief Instruction Set Architecture properties.
@@ -1623,7 +1671,8 @@ class Device : public RuntimeObject {
   typedef enum MemorySegment {
     kNoAtomics = 0,
     kAtomics = 1,
-    kKernArg = 2
+    kKernArg = 2,
+    kUncachedAtomics = 4
   } MemorySegment;
 
   typedef enum CacheState {
@@ -1637,7 +1686,6 @@ class Device : public RuntimeObject {
   enum class VmmAccess {
     kNone           = 0x0,
     kReadOnly       = 0x1,
-    kWriteOnly      = 0x2,
     kReadWrite      = 0x3
   };
 
@@ -1648,6 +1696,11 @@ class Device : public RuntimeObject {
   static constexpr size_t kMGInfoSizePerDevice = kMGSyncDataSize + sizeof(MGSyncInfo);
   static constexpr size_t kSGInfoSize = kMGSyncDataSize;
 
+  // Max Scratch size is based on ISA and thus per device.
+  // Def value is as per GFX9 being the least among supported devices.
+  size_t maxStackSize_ = kMaxStackSize9X;
+  static cl_int gpu_error_; //!< Store the GPU error cause during kernel launch
+
   typedef std::list<CommandQueue*> CommandQueues;
 
   struct BlitProgram : public amd::HeapObject {
@@ -1740,8 +1793,8 @@ class Device : public RuntimeObject {
   //! Allocate a chunk of device memory as a cache for a CL memory object
   virtual device::Memory* createMemory(Memory& owner) const = 0;
 
-  //! Allocate a chunk of device memory without owner class
-  virtual device::Memory* createMemory(size_t size) const = 0;
+  //! Allocate a chunk of device memory with address alignment
+  virtual device::Memory* createMemory(size_t size, size_t alignment = 0) const = 0;
 
   //! Allocate a device sampler object
   virtual bool createSampler(const Sampler&, device::Sampler**) const = 0;
@@ -1890,7 +1943,15 @@ class Device : public RuntimeObject {
    * @param va_addr Virtual Address ptr
    * @param access_flags_ptr Access permissions to be filled
    */
-  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) = 0;
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const = 0;
+
+  /**
+   * Validate Access permisions for a virtual memory object.
+   *
+   * @param va_addr Virtual Address ptr
+   * @param access_flags_ptr Access permissions to be filled
+   */
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const = 0;
 
   /**
    * Free a VA range
@@ -1941,6 +2002,12 @@ class Device : public RuntimeObject {
     return false;
   }
 
+  //! Returns current scratch limit of the device. Valid only on rocm device.
+  virtual size_t ScratchLimitCurrent() const { return 0; }
+
+  //! Sets the current scratch limit of the device. Valid only on rocm device.
+  virtual bool UpdateScratchLimitCurrent(size_t limit) const { return true; }
+
   //! Validate kernel
   virtual bool validateKernel(const amd::Kernel& kernel,
                               const device::VirtualDevice* vdev,
@@ -1962,12 +2029,6 @@ class Device : public RuntimeObject {
     return false;
   };
 
-  // Returns the status of HW event, associated with amd::Event
-  virtual bool IsHwEventReadyForcedWait(
-      const amd::Event& event) const {  //!< AMD event for HW status validation
-    return false;
-  };
-
   virtual void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const {};
 
   virtual const uint32_t getPreferredNumaNode() const { return 0; }
@@ -2040,12 +2101,12 @@ class Device : public RuntimeObject {
   //! Checks if OCL runtime can use hsail for compilation
   bool ValidateHsail();
 
-  bool IpcCreate(void* dev_ptr, size_t* mem_size, void* handle, size_t* mem_offset) const;
+  bool IpcCreate(void* dev_ptr, size_t* mem_size, char* handle, size_t* mem_offset) const;
 
-  bool IpcAttach(const void* handle, size_t mem_size, size_t mem_offset, unsigned int flags,
+  bool IpcAttach(const char* handle, size_t mem_size, size_t mem_offset, unsigned int flags,
                  void** dev_ptr) const;
 
-  bool IpcDetach(void* dev_ptr) const;
+  void IpcDetach(amd::Memory* amd_mem_obj) const;
 
   //! Return context
   amd::Context& context() const { return *context_; }
@@ -2090,10 +2151,7 @@ class Device : public RuntimeObject {
   }
 
   //! Returns the queues that have at least one submitted command
-  std::vector<amd::CommandQueue*> getActiveQueues() {
-     amd::ScopedLock lock(activeQueuesLock_);
-     return std::vector<amd::CommandQueue*>(activeQueues.begin(), activeQueues.end());
-  }
+  std::vector<amd::CommandQueue*> getActiveQueues();
 
   //! Adds the queue to the set of active command queues
   void addToActiveQueues(amd::CommandQueue* commandQueue) {
@@ -2119,11 +2177,20 @@ class Device : public RuntimeObject {
     return nullptr;
   }
 
+  //! Returns stack size set for the device
+  size_t MaxStackSize() const { return maxStackSize_; }
+
 #if defined(__clang__)
 #if __has_feature(address_sanitizer)
   virtual device::UriLocator* createUriLocator() const = 0;
 #endif
 #endif
+
+  static bool IsGPUInError() { return (gpu_error_ != CL_SUCCESS); }
+  static cl_int GetGPUError() { return gpu_error_; }
+
+  bool GetHandleForAddressRange(void* dev_ptr, size_t size, void* handle);
+
  protected:
   //! Enable the specified extension
   char* getExtensionString();
@@ -2156,8 +2223,9 @@ class Device : public RuntimeObject {
   uint64_t stack_size_{1024};       //!< Device stack size
   device::Memory* initial_heap_buffer_;   //!< Initial heap buffer
   uint64_t initial_heap_size_{HIP_INITIAL_DM_SIZE};  //!< Initial device heap size
-  amd::Monitor activeQueuesLock_ {"Guards access to the activeQueues set"};
+  amd::Monitor activeQueuesLock_ {}; //!< Guards access to the activeQueues set
   std::unordered_set<amd::CommandQueue*> activeQueues; //!< The set of active queues
+
  private:
   const Isa *isa_;                //!< Device isa
   bool IsTypeMatching(cl_device_type type, bool offlineDevices);
