@@ -54,7 +54,7 @@ namespace option {
 }  // option
 }
 
-namespace device {
+namespace amd::device {
 class ClBinary;
 class Kernel;
 
@@ -131,6 +131,7 @@ class Program : public amd::HeapObject {
        uint32_t hasGlobalStores_ : 1; //!< Program has writable program scope variables
        uint32_t isHIP_ : 1;           //!< Determine if the program is for HIP
        uint32_t coLoaded_ : 1;        //!< Has the code objected been loaded
+       uint32_t trapHandler_ : 1;     //!< It is a trap handler for debugger
      };
      uint32_t flags_;  //!< Program flags
    };
@@ -193,7 +194,7 @@ class Program : public amd::HeapObject {
 
   //! Build the device program.
   int32_t build(const std::string& sourceCode, const char* origOptions,
-                amd::option::Options* options, const std::vector<std::string>& preCompiledHeaders);
+                amd::option::Options* options);
 
   //! Load the device program.
   bool load();
@@ -263,6 +264,9 @@ class Program : public amd::HeapObject {
   //! Return TRUE if the program has been loaded
   bool isCodeObjectLoaded() const { return coLoaded_; }
 
+  //! Returns TRUE if the program is a trap handler for debugger support
+  bool isTrapHandler() const { return trapHandler_; }
+
 #if defined(USE_COMGR_LIBRARY)
   amd_comgr_metadata_node_t metadata() const { return metadata_; }
 
@@ -315,11 +319,10 @@ class Program : public amd::HeapObject {
   *
   *  \return True if we successefully compiled a GPU program
   */
-  virtual bool compileImpl(
-      const std::string& sourceCode,  //!< the program's source code
-      const std::vector<const std::string*>& headers, const char** headerIncludeNames,
-      amd::option::Options* options,                      //!< compile options's object
-      const std::vector<std::string>& preCompiledHeaders  //!< precompiled headers
+  virtual bool compileImpl(const std::string& sourceCode,  //!< the program's source code
+                           const std::vector<const std::string*>& headers,
+                           const char** headerIncludeNames,
+                           amd::option::Options* options  //!< compile options's object
   );
 
   //! Link the device program.
@@ -402,8 +405,7 @@ class Program : public amd::HeapObject {
 
   //! Compile the device program with LC path
   bool compileImplLC(const std::string& sourceCode, const std::vector<const std::string*>& headers,
-                     const char** headerIncludeNames, amd::option::Options* options,
-                     const std::vector<std::string>& preCompiledHeaders);
+                     const char** headerIncludeNames, amd::option::Options* options);
 
   //! Compile the device program with HSAIL path
   bool compileImplHSAIL(const std::string& sourceCode,
@@ -443,10 +445,6 @@ class Program : public amd::HeapObject {
     const size_t size, const amd_comgr_data_kind_t type,
     const char* name, amd_comgr_data_set_t* dataSet);
 
-  //! Add precompiled headers to the data set
-  amd_comgr_status_t addPreCompiledHeader(amd_comgr_data_set_t* dataSet,
-                                          const std::vector<std::string>& preCompiledHeaders);
-
   //! Create action for the specified language, target and options
   amd_comgr_status_t createAction(const amd_comgr_language_t oclver,
     const std::vector<std::string>& options, amd_comgr_action_info_t* action,
@@ -456,13 +454,13 @@ class Program : public amd::HeapObject {
   bool linkLLVMBitcode(const amd_comgr_data_set_t inputs,
     const std::vector<std::string>& options,
     amd::option::Options* amdOptions, amd_comgr_data_set_t* output,
-    char* binaryData[] = nullptr, size_t* binarySize = nullptr,
-    const bool link_dev_libs = true);
+    char* binaryData[] = nullptr, size_t* binarySize = nullptr);
 
   //! Create the bitcode of the compiled input dataset
   bool compileToLLVMBitcode(const amd_comgr_data_set_t compileInputs,
     const std::vector<std::string>& options, amd::option::Options* amdOptions,
-    char* binaryData[], size_t* binarySize);
+    char* binaryData[], size_t* binarySize,
+    const bool link_dev_libs = true);
 
   //! Compile and create the excutable of the input dataset
   bool compileAndLinkExecutable(const amd_comgr_data_set_t inputs,
@@ -500,4 +498,4 @@ class ComgrBinaryData {
 
 #endif
 
-} // namespace device
+} // namespace amd::device
