@@ -36,7 +36,7 @@
 #include "device/pal/paldefs.hpp"
 #include "device/pal/palsettings.hpp"
 #include "device/pal/palappprofile.hpp"
-#include "device/pal/palgpuopen.hpp"
+#include "device/pal/palcapturemgr.hpp"
 #include "device/pal/palsignal.hpp"
 #include "acl.h"
 #include "memory"
@@ -92,8 +92,7 @@ class NullDevice : public amd::Device {
   //! Just returns NULL for the dummy device
   virtual device::Memory* createMemory(amd::Memory& owner) const { return nullptr; }
   //! Just returns NULL for the dummy device
-  virtual device::Memory* createMemory(size_t size) const { return nullptr; }
-
+  virtual device::Memory* createMemory(size_t size, size_t alignment = 0) const { return nullptr; }
   //! Sampler object allocation
   virtual bool createSampler(const amd::Sampler& owner,  //!< abstraction layer sampler object
                              device::Sampler** sampler   //!< device sampler object
@@ -151,7 +150,11 @@ class NullDevice : public amd::Device {
     return true;
   }
 
-  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) {
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const {
+    return true;
+  }
+
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const {
     return true;
   }
 
@@ -228,7 +231,8 @@ class Sampler : public device::Sampler {
   virtual ~Sampler();
 
   //! Creates a device sampler from the OCL sampler state
-  bool create(uint32_t oclSamplerState  //!< OCL sampler state
+  bool create(uint32_t oclSamplerState,  //!< OCL sampler state
+              const uint addressMode[3]  //!< Address modes in X, Y and Z
   );
 
   //! Creates a device sampler from the OCL sampler state
@@ -255,7 +259,7 @@ class Device : public NullDevice {
     amd::Monitor queue_lock_;       //!< Queue lock for access
     AqlPacketMgmt aql_packet_mgmt_; //!< AQL packets management class for debugger support
     QueueRecycleInfo() : counter_(1), engineType_(Pal::EngineTypeCompute), index_(0),
-          queue_lock_("Queue lock for sharing", true) {}
+          queue_lock_(true) /* Queue lock for sharing */ {}
 
     //! Returns the aql packet list
     uintptr_t AqlPacketList() const {
@@ -399,9 +403,7 @@ class Device : public NullDevice {
   //! Memory allocation
   virtual device::Memory* createMemory(amd::Memory& owner  //!< abstraction layer memory object
                                        ) const;
-  virtual device::Memory* createMemory(size_t size    //!< Size of memory allocation
-                                       ) const;
-
+  virtual device::Memory* createMemory(size_t size, size_t alignment = 0) const;
   //! Sampler object allocation
   virtual bool createSampler(const amd::Sampler& owner,  //!< abstraction layer sampler object
                              device::Sampler** sampler   //!< device sampler object
@@ -535,6 +537,7 @@ class Device : public NullDevice {
   void fillHwSampler(uint32_t state,                       //!< Sampler's OpenCL state
                      void* hwState,                        //!< Sampler's HW state
                      uint32_t hwStateSize,                 //!< Size of sampler's HW state
+                     const uint* addressMode,              //!< Address modes in X, Y and Z
                      uint32_t mipFilter = CL_FILTER_NONE,  //!< Mip filter
                      float minLod = 0.f,                   //!< Min level of detail
                      float maxLod = CL_MAXFLOAT            //!< Max level of detail
@@ -561,7 +564,8 @@ class Device : public NullDevice {
 
   //! Set/Get memory access set by the app
   virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags);
-  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr);
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const;
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const;
 
   virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags, void* shareableHandle);
 
@@ -582,7 +586,8 @@ class Device : public NullDevice {
   //! Allow access for peer device
   bool deviceAllowAccess(void* dst) const;
 
-  RgpCaptureMgr* rgpCaptureMgr() const { return rgpCaptureMgr_; }
+  //! Returns a handle to the capture manager (RGP or UberTrace)
+  ICaptureMgr* captureMgr() const { return captureMgr_; }
 
   //! Update free memory for OCL extension
   void updateAllocedMemory(Pal::GpuHeap heap,  //!< PAL GPU heap for update
@@ -670,6 +675,17 @@ class Device : public NullDevice {
   //! Allocates hidden heap for device memory allocations
   void HiddenHeapAlloc(const VirtualGPU& gpu);
 
+  Pal::gpusize GetMaxFrameBuffer() const {
+    return maxFrameBufferAllocation_;
+  }
+
+  Pal::gpusize TotalAlloc() const {
+    Pal::gpusize local = allocedMem[Pal::GpuHeapLocal] - resourceCache().persistentCacheSize();
+    Pal::gpusize invisible = allocedMem[Pal::GpuHeapInvisible] - resourceCache().lclCacheSize();
+    Pal::gpusize total_alloced = local + invisible;
+    return total_alloced;
+  }
+
  private:
   static void PAL_STDCALL PalDeveloperCallback(void* pPrivateData, const Pal::uint32 deviceIndex,
                                                Pal::Developer::CallbackType type, void* pCbData);
@@ -735,15 +751,16 @@ class Device : public NullDevice {
   Memory* globalScratchBuf_;             //!< Global scratch buffer
   SrdManager* srdManager_;               //!< SRD manager object
   static AppProfile appProfile_;         //!< application profile
-  mutable bool freeCPUMem_;              //!< flag to mark GPU free SVM CPU mem
+  mutable bool freeCPUMem_ = false;      //!< flag to mark GPU free SVM CPU mem
   Pal::DeviceProperties properties_;     //!< PAL device properties
   Pal::IDevice* device_;                 //!< PAL device object
   mutable std::atomic<Pal::gpusize>
       allocedMem[Pal::GpuHeap::GpuHeapCount];    //!< Free memory counter
   std::unordered_set<Resource*>* resourceList_;  //!< Active resource list
-  RgpCaptureMgr* rgpCaptureMgr_;                 //!< RGP capture manager
+  ICaptureMgr* captureMgr_;                      //!< RGP/UberTrace capture manager
   Pal::GpuMemoryHeapProperties
       heaps_[Pal::GpuHeapCount];         //!< Information about heaps, returned from PAL
+  Pal::gpusize maxFrameBufferAllocation_; //!< To reserve some memory in frame buffer
   std::map<Pal::IQueue*, QueueRecycleInfo*> queue_pool_;  //!< Pool of PAL queues for recycling
   amd::Program* trap_handler_ = nullptr;  //!< Trap handler program for debugger setup
 };
