@@ -26,6 +26,7 @@
 #include "device/device.hpp"
 #include "device/pal/paldevice.hpp"
 #include "device/pal/palblit.hpp"
+#include "utils/flags.hpp"
 
 #ifdef _WIN32
 #include <d3d10_1.h>
@@ -113,13 +114,19 @@ bool Memory::create(Resource::MemoryType memType, Resource::CreateParams* params
         dev().info().largeBar_) {
       memType = Persistent;
     }
+
+    if (amd::IS_HIP && dev().settings().apuSystem_) {
+      Pal::gpusize totalAlloc = dev().TotalAlloc();
+      if (memType == Local && totalAlloc > dev().GetMaxFrameBuffer()) {
+        memType = RemoteUSWC;
+      }
+    }
     // Create a resource in PAL
     result = Resource::create(memType, params, forceLinear);
     if (!result) {
       size_t freeMemory[2];
       // if requested memory is greater than available then exit the loop
       dev().globalFreeMemory(freeMemory);
-
       // Local to Persistent
       if (memoryType() == Local) {
         // For dgpu freeMemory[0] reports a sum of visible+invisible fb
@@ -1024,8 +1031,10 @@ Memory* Buffer::createBufferView(amd::Memory& subBufferOwner) const {
 // ================================================================================================
 bool Buffer::ExportHandle(void* handle) const {
   Pal::GpuMemoryExportInfo exportInfo = {};
+#if IS_WINDOWS
   // Set default flags in case they are not provided by application
   exportInfo.accessFlags = GENERIC_READ | GENERIC_WRITE;
+#endif
   *reinterpret_cast<Pal::OsExternalHandle*>(handle) = iMem()->ExportExternalHandle(exportInfo);
   return true;
 }
