@@ -43,7 +43,7 @@
 #include <iostream>
 #include <cmath>
 
-namespace pal {
+namespace amd::pal {
 
 // ================================================================================================
 Pal::Result GpuMemoryReference::MakeResident() const {
@@ -82,7 +82,8 @@ GpuMemoryReference* GpuMemoryReference::Create(const Device& dev,
       return nullptr;
     }
   }
-  if (!createInfo.flags.sdiExternal) {
+  memRef->va_range_ = createInfo.flags.virtualAlloc;
+  if (!createInfo.flags.sdiExternal && !createInfo.flags.virtualAlloc) {
     // Update free memory size counters
     dev.updateAllocedMemory(memRef->gpuMem_->Desc().heaps[0], memRef->gpuMem_->Desc().size, false);
   }
@@ -255,29 +256,32 @@ GpuMemoryReference::~GpuMemoryReference() {
   if (nullptr == iMem()) {
     return;
   }
-  if (gpu_ == nullptr) {
-    Device::ScopedLockVgpus lock(device_);
-    // Release all memory objects on all virtual GPUs
-    for (uint idx = 1; idx < device_.vgpus().size(); ++idx) {
-      device_.vgpus()[idx]->releaseMemory(this);
+  // Memory tracking per queue is disabled if alwaysResident is enabled. Thus, runtime can skip
+  // updating residency state per every queue
+  if (!device_.settings().alwaysResident_) {
+    if (gpu_ == nullptr) {
+      Device::ScopedLockVgpus lock(device_);
+      // Release all memory objects on all virtual GPUs
+      for (uint idx = 1; idx < device_.vgpus().size(); ++idx) {
+        device_.vgpus()[idx]->releaseMemory(this);
+      }
+    } else {
+      amd::ScopedLock l(gpu_->execution());
+      gpu_->releaseMemory(this);
+    }
+    if (device_.vgpus().size() != 0) {
+      assert(device_.vgpus()[0] == device_.xferQueue() && "Wrong transfer queue!");
+      // Lock the transfer queue, since it's not handled by ScopedLockVgpus
+      amd::ScopedLock k(device_.xferMgr().lockXfer());
+      device_.vgpus()[0]->releaseMemory(this);
     }
-  } else {
-    amd::ScopedLock l(gpu_->execution());
-    gpu_->releaseMemory(this);
   }
-  if (device_.vgpus().size() != 0) {
-    assert(device_.vgpus()[0] == device_.xferQueue() && "Wrong transfer queue!");
-    // Lock the transfer queue, since it's not handled by ScopedLockVgpus
-    amd::ScopedLock k(device_.xferMgr().lockXfer());
-    device_.vgpus()[0]->releaseMemory(this);
-  }
-
   // Destroy PAL object if it's not a suballocation
   if (cpuAddress_ != nullptr) {
     iMem()->Unmap();
   }
   if (!(iMem()->Desc().flags.isShared || iMem()->Desc().flags.isExternal ||
-        iMem()->Desc().flags.isExternPhys)) {
+        iMem()->Desc().flags.isExternPhys || va_range_)) {
     // Update free memory size counters
     device_.updateAllocedMemory(iMem()->Desc().heaps[0], iMem()->Desc().size, true);
   }
@@ -788,7 +792,7 @@ bool Resource::CreateInterop(CreateParams* params) {
 
     if (!dev().resGLAssociate(oglRes->glPlatformContext_, oglRes->handle_, glType_,
                               &openInfo.hExternalResource, &glInteropMbRes_, &offset_, desc_.format_
-#ifdef ATI_OS_WIN
+#if IS_WINDOWS
                               ,
                               openInfo.doppDesktopInfo
 #endif
@@ -804,6 +808,7 @@ bool Resource::CreateInterop(CreateParams* params) {
     if (vparams->handle_) {
       openInfo.hExternalResource = vparams->handle_;
     } else if (vparams->name_) {
+#if IS_WINDOWS
       Pal::ExternalHandleInfo eHandleInfo = {};
       eHandleInfo.objectType = Pal::ExternalObjectType::Allocation;
       eHandleInfo.pNtObjectName = reinterpret_cast<const wchar_t*>(vparams->name_);
@@ -815,10 +820,13 @@ bool Resource::CreateInterop(CreateParams* params) {
           dev().iDev()->OpenExternalHandleFromName(eHandleInfo, &openInfo.hExternalResource)) {
         return false;
       }
+#else
+      return false;
+#endif
     }
     openInfo.flags.ntHandle = vparams->nt_handle_;
   }
-#ifdef ATI_OS_WIN
+#if IS_WINDOWS
   else {
     D3DInteropParams* d3dRes = reinterpret_cast<D3DInteropParams*>(params);
     openInfo.hExternalResource = d3dRes->handle_;
@@ -879,7 +887,7 @@ bool Resource::CreateInterop(CreateParams* params) {
       switch (misc) {
         case 1:  // NV12 or P010 formats
           switch (layer) {
-            case -1:
+            case std::numeric_limits<uint>::max():
             case 0:
               break;
             case 1:
@@ -895,7 +903,7 @@ bool Resource::CreateInterop(CreateParams* params) {
           break;
         case 2:  // YV12 format
           switch (layer) {
-            case -1:
+            case std::numeric_limits<uint>::max():
             case 0:
               break;
             case 1:
@@ -1040,12 +1048,7 @@ bool Resource::CreateInterop(CreateParams* params) {
     //! and OGL decompresses 24bit DEPTH into D24S8 for OGL compatibility
     if ((desc().format_.image_channel_order == CL_DEPTH_STENCIL) &&
         (desc().format_.image_channel_data_type == CL_UNORM_INT24)) {
-      if (dev().settings().gfx10Plus_) {
-        hwState_[1] = (hwState_[1] & ~0x1ff00000) | 0x08d00000;
-      } else {
-        hwState_[1] &= ~0x3c000000;
-        hwState_[1] = (hwState_[1] & ~0x3f00000) | 0x1400000;
-      }
+      hwState_[1] = (hwState_[1] & ~0x1ff00000) | 0x08d00000;
     }
     hwState_[8] = GetHSAILImageFormatType(desc().format_);
     hwState_[9] = GetHSAILImageOrderType(desc().format_);
@@ -1194,6 +1197,7 @@ bool Resource::CreateSvm(CreateParams* params, Pal::gpusize svmPtr) {
                                                   createInfo.pReservedGpuVaOwner, &subOffset_);
     if (memRef_ == nullptr) {
       createInfo.alignment = dev().properties().gpuMemoryProperties.fragmentSize;
+      createInfo.size = amd::alignUp (createInfo.size, createInfo.alignment);
       memRef_ = GpuMemoryReference::Create(dev(), createInfo);
     }
   }
@@ -1250,7 +1254,7 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
 
   // Force remote allocation if it was requested in the settings
   if (dev().settings().remoteAlloc_ && ((memoryType() == Local) || (memoryType() == Persistent))) {
-    if (dev().settings().apuSystem_ && dev().settings().viPlus_) {
+    if (dev().settings().apuSystem_) {
       desc_.type_ = Remote;
     } else {
       desc_.type_ = RemoteUSWC;
@@ -1310,8 +1314,8 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
         (nullptr != params->owner_->getSvmPtr())) {
       svmPtr = reinterpret_cast<Pal::gpusize>(params->owner_->getSvmPtr());
       desc_.SVMRes_ = true;
-      svmPtr = (svmPtr == 1) ? 0 : svmPtr;
       desc_.reserved_va_ = (svmPtr == 1) ? false : true;
+      svmPtr = (svmPtr == 1) ? 0 : svmPtr;
       if (params->owner_->getMemFlags() & CL_MEM_SVM_ATOMICS) {
         desc_.gl2CacheDisabled_ = true;
       }
@@ -1324,7 +1328,9 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
   Pal::GpuMemoryCreateInfo createInfo = {};
   createInfo.size = desc().width_ * elementSize_;
   createInfo.size = amd::alignUp(createInfo.size, MaxGpuAlignment);
-  createInfo.alignment = desc().scratch_ ? 64 * Ki : MaxGpuAlignment;
+  createInfo.alignment = (params && params->alignment_ != 0)
+                         ? params->alignment_
+                         : (desc().scratch_ ? 64 * Ki : MaxGpuAlignment);
   createInfo.vaRange = Pal::VaRange::Default;
   createInfo.priority = Pal::GpuMemPriority::Normal;
 
@@ -1337,9 +1343,10 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
     createInfo.flags.busAddressable = true;
   } else if (memoryType() == VaRange) {
     createInfo.flags.virtualAlloc = true;
+    createInfo.vaRange = Pal::VaRange::Svm;
     if (params->owner_->getSvmPtr() != nullptr) {
-      createInfo.flags.startVaHintFlag = true;
-      createInfo.startVaHint = reinterpret_cast<Pal::gpusize>(params->owner_->getSvmPtr());
+      createInfo.flags.useReservedGpuVa = true;
+      createInfo.pReservedGpuVaOwner = params->svmBase_->iMem();
     }
   }
 
@@ -1362,7 +1369,16 @@ bool Resource::create(MemoryType memType, CreateParams* params, bool forceLinear
     mapCount_++;
   }
   if (memoryType() == VaRange) {
-    params->owner_->setSvmPtr(reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr));
+    void* gpu_virt_addr = reinterpret_cast<void*>(memRef_->iMem()->Desc().gpuVirtAddr);
+
+    // if svm_ptr was not set, set the gpu_virt_addr
+    if (params->owner_->getSvmPtr() == nullptr) {
+      params->owner_->setSvmPtr(gpu_virt_addr);
+    } else if (params->owner_->getSvmPtr() != gpu_virt_addr) {
+      LogPrintfError("Cannot get different ptrs in va_range, svm_ptr: %p new_svm_ptr:%p \n",
+                     params->owner_->getSvmPtr(), gpu_virt_addr);
+      return false;
+    }
   }
   return true;
 }
@@ -1380,13 +1396,12 @@ void Resource::free() {
   // and resource can be reused on another async queue without a wait on a busy operation
   if (wait) {
     if (memRef_->gpu_ == nullptr) {
-      Device::ScopedLockVgpus lock(dev());
+      amd::ScopedLock l(dev().vgpusAccess());
       // Release all memory objects on all virtual GPUs
       for (uint idx = 1; idx < dev().vgpus().size(); ++idx) {
         dev().vgpus()[idx]->waitForEvent(&events_[idx]);
       }
     } else {
-      amd::ScopedLock l(memRef_->gpu_->execution());
       memRef_->gpu_->waitForEvent(&events_[memRef_->gpu_->index()]);
     }
   } else {
@@ -1474,7 +1489,6 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
   GpuEvent event;
   EngineType activeEngineID = gpu.engineID_;
   static const bool waitOnBusyEngine = true;
-  assert(!(desc().cardMemory_ && dstResource.desc().cardMemory_) && "Unsupported configuraiton!");
   uint64_t gpuMemoryOffset = 0;
   uint64_t gpuMemoryRowPitch = 0;
   uint64_t imageOffsetx = 0;
@@ -1501,7 +1515,7 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
     // Make sure linear pitch in bytes is 4 bytes aligned
     if (((gpuMemoryRowPitch % 4) != 0) ||
         // another DRM restriciton... SI has 4 pixels
-        (gpuMemoryOffset % 4 != 0) || (dev().settings().sdamPageFaultWar_ && (imageOffsetx != 0))) {
+        (gpuMemoryOffset % 4 != 0) || (imageOffsetx != 0)) {
       return false;
     }
   }
@@ -1511,21 +1525,21 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
        (size[0] < dev().settings().cpDmaCopySizeMax_));
   if (cp_dma) {
     // Make sure compute is done before CP DMA start
-    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::KernelToCopy);
   } else {
     gpu.releaseGpuMemoryFence();
     gpu.engineID_ = SdmaEngine;
+
+    if (gpu.validateSdmaOverlap(*this, dstResource)) {
+      // Note: PAL should insert a NOP into the command buffer for synchronization
+      gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::CopyToCopy);
+    }
   }
 
   // Wait for the resources, since runtime may use async transfers
   wait(gpu, waitOnBusyEngine);
   dstResource.wait(gpu, waitOnBusyEngine);
 
-  if (gpu.validateSdmaOverlap(*this, dstResource)) {
-    // Note: PAL should insert a NOP into the command buffer for synchronization
-    gpu.addBarrier();
-  }
-
   Pal::ImageLayout imgLayout = {};
   gpu.eventBegin(gpu.engineID_);
   gpu.queue(gpu.engineID_).addCmdMemRef(memRef());
@@ -1625,7 +1639,7 @@ bool Resource::partialMemCopyTo(VirtualGPU& gpu, const amd::Coord3D& srcOrigin,
 
   if (cp_dma) {
     // Make sure CP dma is done
-    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency);
+    gpu.addBarrier(RgpSqqtBarrierReason::MemDependency, BarrierType::CopyToKernel);
   }
 
   gpu.eventEnd(gpu.engineID_, event);
@@ -1697,7 +1711,7 @@ bool Resource::hostWrite(VirtualGPU* gpu, const void* hostPtr, const amd::Coord3
     dst = static_cast<void*>(static_cast<char*>(dst) + origin[0]);
 
     // Copy memory
-    amd::Os::fastMemcpy(dst, hostPtr, copySize);
+    std::memcpy(dst, hostPtr, copySize);
   } else {
     size_t dstOffsBase = origin[0] * elementSize_;
 
@@ -1725,7 +1739,7 @@ bool Resource::hostWrite(VirtualGPU* gpu, const void* hostPtr, const amd::Coord3
       // Copy memory line by line
       for (size_t row = 0; row < size[1]; ++row) {
         // Copy memory
-        amd::Os::fastMemcpy((reinterpret_cast<address>(dst) + dstOffs),
+        std::memcpy((reinterpret_cast<address>(dst) + dstOffs),
                             (reinterpret_cast<const_address>(hostPtr) + srcOffs),
                             size[0] * elementSize_);
 
@@ -1767,7 +1781,7 @@ bool Resource::hostRead(VirtualGPU* gpu, void* hostPtr, const amd::Coord3D& orig
     src = static_cast<void*>(static_cast<char*>(src) + origin[0]);
 
     // Copy memory
-    amd::Os::fastMemcpy(hostPtr, src, copySize);
+    std::memcpy(hostPtr, src, copySize);
   } else {
     size_t srcOffsBase = origin[0] * elementSize_;
 
@@ -1795,9 +1809,9 @@ bool Resource::hostRead(VirtualGPU* gpu, void* hostPtr, const amd::Coord3D& orig
       // Copy memory line by line
       for (size_t row = 0; row < size[1]; ++row) {
         // Copy memory
-        amd::Os::fastMemcpy((reinterpret_cast<address>(hostPtr) + dstOffs),
-                            (reinterpret_cast<const_address>(src) + srcOffs),
-                            size[0] * elementSize_);
+        std::memcpy((reinterpret_cast<address>(hostPtr) + dstOffs),
+                    (reinterpret_cast<const_address>(src) + srcOffs),
+                    size[0] * elementSize_);
 
         srcOffs += desc().pitch_ * elementSize_;
         dstOffs += rowPitch;
@@ -1936,7 +1950,7 @@ bool Resource::isPersistentDirectMap(bool writeMap) const {
   if (directMap && desc().tiled_) {
     // Latest HW does have tiling apertures
     directMap = false;
-  } 
+  }
   if (memoryType() == View) {
     directMap = viewOwner_->isPersistentDirectMap(writeMap);
   }
@@ -2196,7 +2210,8 @@ bool MemorySubAllocator::Free(amd::Monitor* monitor, GpuMemoryReference* ref, Pa
 
     it->second->Free(offset);
     // If this suballocator empty, then release memory chunk
-    if (it->second->IsEmpty()) {
+    // while keeping at least one chunk available, if retain_final_chunk is true
+    if (it->second->IsEmpty() && (!(retain_final_chunk_ && heaps_.size() == 1) || !amd::IS_HIP)) {
       delete it->second;
       heaps_.erase(it);
       release_mem = true;
@@ -2219,7 +2234,10 @@ bool ResourceCache::addGpuMemory(Resource::Descriptor* desc, GpuMemoryReference*
   size_t size = ref->iMem()->Desc().size;
 
   // Check if runtime can free suballocation
-  if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
+  if (desc->type_ == Resource::VaRange) {
+    // We do no sub allocate VA Range.
+    result = false;
+  } else if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
     result = mem_sub_alloc_local_.Free(&lockCacheOps_, ref, offset);
   } else if ((desc->type_ == Resource::Local) && desc->SVMRes_) {
     result = mem_sub_alloc_coarse_.Free(&lockCacheOps_, ref, offset);
@@ -2276,7 +2294,10 @@ GpuMemoryReference* ResourceCache::findGpuMemory(Resource::Descriptor* desc, Pal
   GpuMemoryReference* ref = nullptr;
 
   // Check if the runtime can suballocate memory
-  if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
+  if (desc->type_ == Resource::VaRange) {
+    // Do not use suballocator for VA_Range.
+    return nullptr;
+  } else if ((desc->type_ == Resource::Local) && !desc->SVMRes_) {
     ref = mem_sub_alloc_local_.Allocate(size, alignment, reserved_va, offset);
   } else if ((desc->type_ == Resource::Local) && desc->SVMRes_) {
     ref = mem_sub_alloc_coarse_.Allocate(size, alignment, reserved_va, offset);
@@ -2365,4 +2386,4 @@ void ResourceCache::removeLast() {
   entry.second->release();
 }
 
-}  // namespace pal
+}  // namespace amd::pal
