@@ -70,6 +70,7 @@ class GpuMemoryReference : public amd::ReferenceCountedObject {
   const Device& device_;     //!< GPU device
   //! @note: This field is necessary for the thread safe release only
   VirtualGPU* gpu_;  //!< Resource will be used only on this queue
+  bool va_range_ = false; //!< Resource is a VA range(@note: PAL doesn't provide this info)
 
  protected:
   //! Default destructor
@@ -104,7 +105,9 @@ class Resource : public amd::HeapObject {
     VirtualGPU* gpu_;          //!< Resource won't be shared between multiple queues
     const Resource* svmBase_;  //!< SVM base for MGPU allocations
     bool interprocess_;        //!< Ressource can be used in the interprocess communication
-    CreateParams() : owner_(nullptr), gpu_(nullptr), svmBase_(nullptr), interprocess_(false) {}
+    size_t alignment_;         //!< allocation address alignment
+    CreateParams() : owner_(nullptr), gpu_(nullptr), svmBase_(nullptr), interprocess_(false),
+                     alignment_(0) {}
   };
 
   struct PinnedParams : public CreateParams {
@@ -535,7 +538,8 @@ typedef Util::BuddyAllocator<Device> MemBuddyAllocator;
 
 class MemorySubAllocator : public amd::HeapObject {
  public:
-  MemorySubAllocator(Device* device) : device_(device) {}
+  MemorySubAllocator(Device* device, bool retain_final_chunk = false)
+      : device_(device), retain_final_chunk_(retain_final_chunk) {}
 
   ~MemorySubAllocator();
 
@@ -553,6 +557,7 @@ class MemorySubAllocator : public amd::HeapObject {
 
   Device* device_;
   std::unordered_map<GpuMemoryReference*, MemBuddyAllocator*> heaps_;
+  bool retain_final_chunk_;
 };
 
 class CoarseMemorySubAllocator : public MemorySubAllocator {
@@ -564,7 +569,8 @@ class CoarseMemorySubAllocator : public MemorySubAllocator {
 
 class FineMemorySubAllocator : public MemorySubAllocator {
  public:
-  FineMemorySubAllocator(Device* device) : MemorySubAllocator(device) {}
+  FineMemorySubAllocator(Device* device)
+      : MemorySubAllocator(device, true /*retain_final_chunk*/) {}
 
   bool CreateChunk(const Pal::IGpuMemory* reserved_va) override;
 };
@@ -579,7 +585,7 @@ class ResourceCache : public amd::HeapObject {
  public:
   //! Default constructor
   ResourceCache(Device* device, size_t cacheSizeLimit)
-      : lockCacheOps_("PAL resource cache", true),
+      : lockCacheOps_(true), /* PAL resource cache */
         cacheSize_(0),
         lclCacheSize_(0),
         persistentCacheSize_(0),
