@@ -65,21 +65,17 @@ class Settings : public device::Settings {
       uint use64BitPtr_ : 1;            //!< Use 64bit pointers on GPU
       uint force32BitOcl20_ : 1;        //!< Force 32bit apps to take CLANG/HSAIL path on GPU
       uint imageDMA_ : 1;               //!< Enable direct image DMA transfers
-      uint viPlus_ : 1;                 //!< VI and post VI features
-      uint aiPlus_ : 1;                 //!< AI and post AI features
-      uint gfx10Plus_ : 1;              //!< gfx10 and post gfx10 features
       uint threadTraceEnable_ : 1;      //!< Thread trace enable
       uint svmAtomics_ : 1;             //!< SVM device atomics
       uint svmFineGrainSystem_ : 1;     //!< SVM fine grain system support
       uint useDeviceQueue_ : 1;         //!< Submit to separate device queue
-      uint sdamPageFaultWar_ : 1;       //!< SDMA page fault workaround
       uint rgpSqttWaitIdle_ : 1;        //!< Wait for idle after SQTT trace
       uint rgpSqttForceDisable_ : 1;    //!< Disables SQTT
       uint enableHwP2P_ : 1;            //!< Forces HW P2P path for testing
       uint imageBufferWar_ : 1;         //!< Image buffer workaround for Gfx10
       uint disableSdma_ : 1;            //!< Disable SDMA support
       uint alwaysResident_ : 1;         //!< Make resources resident at allocation time
-      uint reserved_ : 10;
+      uint reserved_ : 13;
     };
     uint value_;
   };
@@ -98,7 +94,6 @@ class Settings : public device::Settings {
   uint hostMemDirectAccess_;     //!< Enables direct access to the host memory
   uint numScratchWavesPerCu_;    //!< Maximum number of waves when scratch is enabled
   size_t xferBufSize_;           //!< Transfer buffer size for image copy optimization
-  size_t stagedXferSize_;        //!< Staged buffer size
   size_t pinnedXferSize_;        //!< Pinned buffer size for transfer
   size_t pinnedMinXferSize_;     //!< Minimal buffer size for pinned transfer
   size_t cpDmaCopySizeMax_;      //!< Threshold for CP DMA path in copy
