@@ -82,13 +82,13 @@ public:
   Timestamp*    ts_;      //!< Timestamp object associated with the signal
   HwQueueEngine engine_;  //!< Engine used with this signal
   amd::Monitor  lock_;    //!< Signal lock for update
-  bool isPacketDispatch_; //!< True if the packet associated with the signal is dispatch
 
   typedef union {
     struct {
-      uint32_t          done_            :  1; //!< True if signal is done
-      uint32_t          forceHostWait_   :  1; //!< Force Host Wait for dependency signals
-      uint32_t          reserved_        : 30;
+      uint32_t  done_            :  1; //!< True if signal is done
+      uint32_t  isPacketDispatch_:  1; //!< True if the packet, used with the signal, is dispatch
+      uint32_t  interrupt_       :  1; //!< True if the signal will trigger an interrupt
+      uint32_t  reserved_        : 29;
     };
     uint32_t data_;
   } Flags;
@@ -98,12 +98,11 @@ public:
   ProfilingSignal()
     : ts_(nullptr)
     , engine_(HwQueueEngine::Compute)
-    , lock_("Signal Ops Lock", true)
-    , isPacketDispatch_(false)
+    , lock_(true) /* Signal Ops Lock */
     {
       signal_.handle = 0;
+      flags_.data_ = 0;
       flags_.done_ = true;
-      flags_.forceHostWait_ = true;
     }
 
   virtual ~ProfilingSignal();
@@ -123,7 +122,7 @@ class Sampler : public device::Sampler {
               );
 
  private:
-  void fillSampleDescriptor(hsa_ext_sampler_descriptor_t& samplerDescriptor,
+  void fillSampleDescriptor(hsa_ext_sampler_descriptor_v2_t& samplerDescriptor,
                             const amd::Sampler& sampler) const;
   Sampler& operator=(const Sampler&);
 
@@ -179,11 +178,10 @@ class NullDevice : public amd::Device {
     ShouldNotReachHere();
     return nullptr;
   }
-  device::Memory* createMemory(size_t size) const override {
+  device::Memory* createMemory(size_t size, size_t alignment = 0) const override {
     ShouldNotReachHere();
     return nullptr;
   }
-
   //! Sampler object allocation
   bool createSampler(const amd::Sampler& owner,  //!< abstraction layer sampler object
                      device::Sampler** sampler   //!< device sampler object
@@ -240,11 +238,16 @@ class NullDevice : public amd::Device {
     return false;
   }
 
-  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) override {
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const {
     ShouldNotReachHere();
     return false;
   }
 
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const {
+    ShouldNotReachHere();
+    return true;
+  }
+
   //! Determine if we can use device memory for SVM
   const bool forceFineGrain(amd::Memory* memory) const {
     return (memory->getContext().devices().size() > 1);
@@ -292,7 +295,6 @@ class NullDevice : public amd::Device {
     return false;
   }
 
-  bool IsHwEventReadyForcedWait(const amd::Event& event) const override { return false; }
   void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const override{};
   void ReleaseGlobalSignal(void* signal) const override {}
 
@@ -320,56 +322,13 @@ struct AgentInfo {
   hsa_amd_memory_pool_t fine_grain_pool;
   hsa_amd_memory_pool_t coarse_grain_pool;
   hsa_amd_memory_pool_t kern_arg_pool;
-  hsa_amd_memory_pool_t ext_fine_grain_pool_;
+  hsa_amd_memory_pool_t ext_fine_grain_pool;
 };
 
 //! A HSA device ordinal (physical HSA device)
 class Device : public NullDevice {
  public:
-  //! Transfer buffers
-  class XferBuffers : public amd::HeapObject {
-   public:
-    static const size_t MaxXferBufListSize = 8;
-
-    //! Default constructor
-    XferBuffers(const Device& device, size_t bufSize)
-        : bufSize_(bufSize), acquiredCnt_(0), gpuDevice_(device) {}
-
-    //! Default destructor
-    ~XferBuffers();
-
-    //! Creates the xfer buffers object
-    bool create();
-
-    //! Acquires an instance of the transfer buffers
-    Memory& acquire();
-
-    //! Releases transfer buffer
-    void release(VirtualGPU& gpu,  //!< Virual GPU object used with the buffer
-                 Memory& buffer    //!< Transfer buffer for release
-                 );
-
-    //! Returns the buffer's size for transfer
-    size_t bufSize() const { return bufSize_; }
-
-   private:
-    //! Disable copy constructor
-    XferBuffers(const XferBuffers&);
-
-    //! Disable assignment operator
-    XferBuffers& operator=(const XferBuffers&);
-
-    //! Get device object
-    const Device& dev() const { return gpuDevice_; }
-
-    size_t bufSize_;                  //!< Staged buffer size
-    std::list<Memory*> freeBuffers_;  //!< The list of free buffers
-    std::atomic_uint acquiredCnt_;   //!< The total number of acquired buffers
-    amd::Monitor lock_;               //!< Stgaed buffer acquire/release lock
-    const Device& gpuDevice_;         //!< GPU device object
-  };
-
-  //! Initialise the whole HSA device subsystem (CAL init, device enumeration, etc).
+  //! Initialise the whole HSA device subsystem (init, device enumeration, etc).
   static bool init();
   static void tearDown();
 
@@ -409,8 +368,7 @@ class Device : public NullDevice {
   virtual device::Program* createProgram(amd::Program& owner, amd::option::Options* options = nullptr);
 
   virtual device::Memory* createMemory(amd::Memory& owner) const;
-  virtual device::Memory* createMemory(size_t size) const;
-
+  virtual device::Memory* createMemory(size_t size, size_t alignment = 0) const;
   //! Sampler object allocation
   virtual bool createSampler(const amd::Sampler& owner,  //!< abstraction layer sampler object
                              device::Sampler** sampler   //!< device sampler object
@@ -448,7 +406,6 @@ class Device : public NullDevice {
 
   //! Gets free memory on a GPU device
   virtual bool globalFreeMemory(size_t* freeMemory) const;
-
   virtual void* hostAlloc(size_t size, size_t alignment,
                           MemorySegment mem_seg = MemorySegment::kNoAtomics) const;
 
@@ -473,12 +430,14 @@ class Device : public NullDevice {
                                 amd::MemoryAdvice advice, bool use_cpu = false) const;
   virtual bool GetSvmAttributes(void** data, size_t* data_sizes, int* attributes,
                                 size_t num_attributes, const void* dev_ptr, size_t count) const;
-
+  virtual size_t ScratchLimitCurrent() const final;
+  virtual bool UpdateScratchLimitCurrent(size_t limit) const final;
   virtual void* virtualAlloc(void* req_addr, size_t size, size_t alignment);
   virtual bool virtualFree(void* addr);
 
   virtual bool SetMemAccess(void* va_addr, size_t va_size, VmmAccess access_flags);
-  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr);
+  virtual bool GetMemAccess(void* va_addr, VmmAccess* access_flags_ptr) const;
+  virtual bool ValidateMemAccess(amd::Memory& mem, bool read_write) const { return true; }
 
   virtual bool ExportShareableVMMHandle(amd::Memory& amd_mem_obj, int flags, void* shareableHandle);
 
@@ -491,15 +450,14 @@ class Device : public NullDevice {
 
   virtual bool IsHwEventReady(const amd::Event& event, bool wait = false,
                               uint32_t hip_event_flags = 0) const;
-  virtual bool IsHwEventReadyForcedWait(const amd::Event& event) const;
   virtual void getHwEventTime(const amd::Event& event, uint64_t* start, uint64_t* end) const;
   virtual void ReleaseGlobalSignal(void* signal) const;
 
   //! Allocate host memory in terms of numa policy set by user
-  void* hostNumaAlloc(size_t size, size_t alignment, bool atomics = false) const;
+  void* hostNumaAlloc(size_t size, size_t alignment, MemorySegment mem_seg) const;
 
   //! Allocate host memory from agent info
-  void* hostAgentAlloc(size_t size, const AgentInfo& agentInfo, bool atomics = false) const;
+  void* hostAgentAlloc(size_t size, const AgentInfo& agentInfo, MemorySegment mem_seg) const;
 
   //! Returns transfer engine object
   const device::BlitManager& xferMgr() const { return xferQueue()->blitMgr(); }
@@ -514,12 +472,6 @@ class Device : public NullDevice {
   //! Adds a map target to the cache
   bool addMapTarget(amd::Memory* memory) const;
 
-  //! Returns transfer buffer object
-  XferBuffers& xferWrite() const { return *xferWrite_; }
-
-  //! Returns transfer buffer object
-  XferBuffers& xferRead() const { return *xferRead_; }
-
   //! Returns a ROC memory object from AMD memory object
   roc::Memory* getRocMemory(amd::Memory* mem  //!< Pointer to AMD memory object
                             ) const;
@@ -539,9 +491,6 @@ class Device : public NullDevice {
   // Update the global free memory size
   void updateFreeMemory(size_t size, bool free);
 
-  bool AcquireExclusiveGpuAccess();
-  void ReleaseExclusiveGpuAccess(VirtualGPU& vgpu) const;
-
   //! Returns the lock object for the virtual gpus list
   amd::Monitor& vgpusAccess() const { return vgpusAccess_; }
 
@@ -553,17 +502,22 @@ class Device : public NullDevice {
   VirtualGPU* xferQueue() const;
 
   hsa_amd_memory_pool_t SystemSegment() const { return system_segment_; }
-
+  hsa_amd_memory_pool_t SystemExtSegment() const { return system_ext_segment_; }
   hsa_amd_memory_pool_t SystemCoarseSegment() const { return system_coarse_segment_; }
 
   //! Acquire HSA queue. This method can create a new HSA queue or
   //! share previously created
   hsa_queue_t* acquireQueue(uint32_t queue_size_hint, bool coop_queue = false,
                             const std::vector<uint32_t>& cuMask = {},
-                            amd::CommandQueue::Priority priority = amd::CommandQueue::Priority::Normal);
+                            amd::CommandQueue::Priority priority = amd::CommandQueue::Priority::Normal,
+                            bool managed = false);
 
   //! Release HSA queue
-  void releaseQueue(hsa_queue_t*, const std::vector<uint32_t>& cuMask = {});
+  void releaseQueue(hsa_queue_t*, const std::vector<uint32_t>& cuMask = {},
+                    bool coop_queue = false, bool managed = false);
+
+  hsa_queue_t* AcquireActiveNormalQueue();
+  bool ReleaseActiveNormalQueue(hsa_queue_t* queue);
 
   //! For the given HSA queue, return an existing hostcall buffer or create a
   //! new one. queuePool_ keeps a mapping from HSA queue to hostcall buffer.
@@ -586,6 +540,10 @@ class Device : public NullDevice {
 
   void getGlobalCUMask(std::string cuMaskStr);
 
+  static hsa_status_t BackendErrorCallBackHandler(const hsa_amd_event_t* event, void* data);
+
+  static void RegisterBackendErrorCb();
+
   virtual amd::Memory* GetArenaMemObj(const void* ptr, size_t& offset, size_t size = 0);
 
   const uint32_t getPreferredNumaNode() const { return preferred_numa_node_; }
@@ -598,11 +556,19 @@ class Device : public NullDevice {
   void HiddenHeapAlloc(const VirtualGPU& gpu);
   //! Init hidden heap for device memory allocations
   void HiddenHeapInit(const VirtualGPU& gpu);
-  uint32_t fetchSDMAMask(const device::BlitManager* handle, bool readEngine = true) const;
-  void resetSDMAMask(const device::BlitManager* handle) const;
   void getSdmaRWMasks(uint32_t* readMask, uint32_t* writeMask) const;
   bool isXgmi() const { return isXgmi_; }
 
+  //! Returns the map of code objects to kernels
+  const auto& KernelMap() const { return kernel_map_; }
+  //! Adds a kernel to the kernel map
+  void AddKernel(Kernel& gpuKernel) const;
+  //! Removes a kernel from the kernel map
+  void RemoveKernel(Kernel& gpuKernel) const;
+
+  // Returns the number of allocated normal queues on this device
+  uint32_t NumNormalQueues() const { return num_normal_queues_.load(); }
+
  private:
   bool create();
 
@@ -635,6 +601,8 @@ class Device : public NullDevice {
   hsa_amd_memory_pool_t system_segment_;
   hsa_amd_memory_pool_t system_coarse_segment_;
   hsa_amd_memory_pool_t system_kernarg_segment_;
+  hsa_amd_memory_pool_t system_ext_segment_;
+
   hsa_amd_memory_pool_t gpuvm_segment_;
   hsa_amd_memory_pool_t gpu_fine_grained_segment_;
   hsa_amd_memory_pool_t gpu_ext_fine_grained_segment_;
@@ -646,20 +614,30 @@ class Device : public NullDevice {
   static constexpr bool offlineDevice_ = false;
   VirtualGPU* xferQueue_;  //!< Transfer queue, created on demand
 
-  XferBuffers* xferRead_;   //!< Transfer buffers read
-  XferBuffers* xferWrite_;  //!< Transfer buffers write
   std::atomic<size_t> freeMem_;   //!< Total of free memory available
   mutable amd::Monitor vgpusAccess_;     //!< Lock to serialise virtual gpu list access
   bool hsa_exclusive_gpu_access_;  //!< TRUE if current device was moved into exclusive GPU access mode
   static address mg_sync_;  //!< MGPU grid launch sync memory (SVM location)
 
   struct QueueInfo {
-    int refCount;
-    void* hostcallBuffer_;
+    int refCount;           //! Reference counter. Shows how many time the queue was shared
+    void* hostcallBuffer_;  //! Host call buffer for the HSA queue
   };
 
+  struct QueueCompare {
+    // Customized queue compare operator to make sure the queues are sorted in the creation order
+    bool operator()(hsa_queue_t* lhs, hsa_queue_t* rhs) const {
+      if (DEBUG_HIP_DYNAMIC_QUEUES) {
+        return (lhs->id < rhs->id) ? true : false;
+      } else {
+        return (lhs < rhs) ? true : false;
+      }
+    }
+  };
   //! a vector for keeping Pool of HSA queues with low, normal and high priorities for recycling
-  std::vector<std::map<hsa_queue_t*, QueueInfo>> queuePool_;
+  std::vector<std::map<hsa_queue_t*, QueueInfo, QueueCompare>> queuePool_;
+  amd::Monitor active_queue_access_;                //!< Lock to serialise virtual gpu list access
+  std::atomic<uint32_t> num_normal_queues_{0};      //!< The total number of allocated normal queues
 
   //! returns a hsa queue from queuePool with least refCount and updates the refCount as well
   hsa_queue_t* getQueueFromPool(const uint qIndex);
@@ -670,15 +648,19 @@ class Device : public NullDevice {
                             std::vector<LinkAttrType>* link_attr);
 
   //! Pool of HSA queues with custom CU masks
-  std::vector<std::map<hsa_queue_t*, QueueInfo>> queueWithCUMaskPool_;
+  std::vector<std::map<hsa_queue_t*, QueueInfo, QueueCompare>> queueWithCUMaskPool_;
 
   //! Read and Write mask for device<->host
   uint32_t maxSdmaReadMask_;
   uint32_t maxSdmaWriteMask_;
-  //! Map of SDMA engineId<->stream
-  mutable std::map<uint32_t, const device::BlitManager*> engineAssignMap_;
   bool isXgmi_; //!< Flag to indicate if there is XGMI between CPU<->GPU
 
+  //! Code object to kernel info map (used in the crash dump analysis)
+  mutable std::map<uint64_t, Kernel&> kernel_map_;
+
+  //! Friend function callbackQueue can access and set device class variables.
+  friend void callbackQueue(hsa_status_t status, hsa_queue_t* queue, void* data);
+
  public:
   std::atomic<uint> numOfVgpus_;  //!< Virtual gpu unique index
 
@@ -691,6 +673,9 @@ class Device : public NullDevice {
 #endif
 #endif
 };                                // class roc::Device
+
+void callbackQueue(hsa_status_t status, hsa_queue_t* queue, void* data);
+
 }  // namespace amd::roc
 
 /**
