@@ -23,6 +23,7 @@
 #include "platform/commandqueue.hpp"
 #include "rocdefs.hpp"
 #include "rocdevice.hpp"
+#include "utils/flags.hpp"
 #include "utils/util.hpp"
 #include "hsa/hsa.h"
 #include "hsa/hsa_ext_image.h"
@@ -46,26 +47,39 @@ constexpr static hsa_signal_value_t kInitSignalValueOne = 1;
 constexpr static uint64_t kTimeout100us = 100 * K;
 constexpr static uint64_t kUnlimitedWait = std::numeric_limits<uint64_t>::max();
 
+constexpr static uint64_t kTimeout4Secs = 4 * M;
+
 inline bool WaitForSignal(hsa_signal_t signal, bool active_wait = false) {
+
+  hsa_wait_state_t wait_state = HSA_WAIT_STATE_BLOCKED;
+  if (active_wait) {
+    wait_state = HSA_WAIT_STATE_ACTIVE;
+  }
+
   if (hsa_signal_load_relaxed(signal) > 0) {
-    uint64_t timeout = kTimeout100us;
-    if (active_wait) {
-      timeout = kUnlimitedWait;
+    // When it is blocked wait, we wait in active state for 100 us before proceeding to wait in
+    // blocked state indefinitely.
+    if (!active_wait) {
+      ClPrint(amd::LOG_INFO, amd::LOG_SIG, "Host active wait for Signal = (0x%lx) for %d ns",
+              signal.handle, kTimeout100us);
+      if (hsa_signal_wait_scacquire(signal, HSA_SIGNAL_CONDITION_LT, kInitSignalValueOne,
+                                    kTimeout100us, HSA_WAIT_STATE_ACTIVE) != 0) {
+        if (HIP_SKIP_ABORT_ON_GPU_ERROR && amd::Device::IsGPUInError()) {
+          ClPrint(amd::LOG_INFO, amd::LOG_SIG, "Device not Stable, while waiting for Signal ="
+                  "(0x%lx) for %d ns", signal.handle, kTimeout100us);
+          return true;
+        }
+      }
     }
 
-    ClPrint(amd::LOG_INFO, amd::LOG_SIG, "Host active wait for Signal = (0x%lx) for %d ns",
-            signal.handle, timeout);
-
-    // Active wait with a timeout
-    if (hsa_signal_wait_scacquire(signal, HSA_SIGNAL_CONDITION_LT, kInitSignalValueOne,
-                                  timeout, HSA_WAIT_STATE_ACTIVE) != 0) {
-      ClPrint(amd::LOG_INFO, amd::LOG_SIG, "Host blocked wait for Signal = (0x%lx)",
-              signal.handle);
-
-      // Wait until the completion with CPU suspend
-      if (hsa_signal_wait_scacquire(signal, HSA_SIGNAL_CONDITION_LT, kInitSignalValueOne,
-                                    kUnlimitedWait, HSA_WAIT_STATE_BLOCKED) != 0) {
-        return false;
+    // This is unlimited wait, but we wait for 4 secs and check if the device is
+    // unstable, if so we return, otherwise we continue to wait in the while loop.
+    while (hsa_signal_wait_scacquire(signal, HSA_SIGNAL_CONDITION_LT, kInitSignalValueOne,
+                                     kTimeout4Secs, wait_state) != 0) {
+      if (HIP_SKIP_ABORT_ON_GPU_ERROR && amd::Device::IsGPUInError()) {
+          ClPrint(amd::LOG_INFO, amd::LOG_SIG, "Device not Stable, while waiting for Signal ="
+                  "(0x%lx) for %d ns", signal.handle, kTimeout4Secs);
+        return true;
       }
     }
   }
@@ -97,8 +111,9 @@ class Timestamp : public amd::ReferenceCountedObject {
   std::vector<ProfilingSignal*> signals_; //!< The list of all signals, associated with the TS
   hsa_signal_t callback_signal_;  //!< Signal associated with a callback for possible later update
   amd::Monitor  lock_;            //!< Serialize timestamp update
-  bool        accum_ena_ = false; //!< If TRUE then the accumulation of execution times has started
-  bool        hasHwProfiling_ = false; //!< If TRUE then HwProfiling is enabled for the command
+  bool accum_ena_ = false;        //!< If TRUE then the accumulation of execution times has started
+  bool hasHwProfiling_ = false;   //!< If TRUE then HwProfiling is enabled for the command
+  bool blocking_ = true;          //!< If TRUE callback is blocking
 
   Timestamp(const Timestamp&) = delete;
   Timestamp& operator=(const Timestamp&) = delete;
@@ -162,12 +177,15 @@ class Timestamp : public amd::ReferenceCountedObject {
   VirtualGPU* gpu() const { return gpu_; }
 
   //! Updates the callback signal
-  void SetCallbackSignal(hsa_signal_t callback_signal) {
+  void SetCallbackSignal(hsa_signal_t callback_signal, bool blocking = true) {
     callback_signal_ = callback_signal;
+    blocking_ = blocking;
   }
-
   //! Returns the callback signal
   hsa_signal_t GetCallbackSignal() const { return callback_signal_; }
+
+  //! Return if callback is blocking/non-blocking
+  bool GetBlocking() { return blocking_; }
 };
 
 class VirtualGPU : public device::VirtualDevice {
@@ -250,7 +268,7 @@ class VirtualGPU : public device::VirtualDevice {
 
     //! Finds a free signal for the upcomming operation
     hsa_signal_t ActiveSignal(hsa_signal_value_t init_val = kInitSignalValueOne,
-                              Timestamp* ts = nullptr);
+                              Timestamp* ts = nullptr, bool attach_signal = true);
 
     //! Wait for the curent active signal. Can idle the queue
     bool WaitCurrent() {
@@ -323,7 +341,7 @@ class VirtualGPU : public device::VirtualDevice {
   const Device& dev() const { return roc_device_; }
 
   void profilingBegin(amd::Command& command, bool sdmaProfiling = false);
-  void profilingEnd(amd::Command& command);
+  void profilingEnd(bool clearHwEvent = false);
 
   void updateCommandsState(amd::Command* list) const;
 
@@ -363,11 +381,6 @@ class VirtualGPU : public device::VirtualDevice {
   void submitSvmUnmapMemory(amd::SvmUnmapMemoryCommand& cmd);
   void submitSvmPrefetchAsync(amd::SvmPrefetchAsyncCommand& cmd);
 
-  // { roc OpenCL integration
-  // Added these stub (no-ops) implementation of pure virtual methods,
-  // when integrating HSA and OpenCL branches.
-  // TODO: After inegration, whoever is working on VirtualGPU should write
-  // actual implementation.
   virtual void submitSignal(amd::SignalCommand& cmd) {}
   virtual void submitMakeBuffersResident(amd::MakeBuffersResidentCommand& cmd) {}
 
@@ -377,6 +390,7 @@ class VirtualGPU : public device::VirtualDevice {
   virtual void submitExternalSemaphoreCmd(amd::ExternalSemaphoreCmd& cmd){}
 
   virtual address allocKernelArguments(size_t size, size_t alignment) final;
+  virtual void ReleaseAllHwQueues() final;
   virtual void ReleaseHwQueue() final;
 
   /**
@@ -389,6 +403,7 @@ class VirtualGPU : public device::VirtualDevice {
 
   hsa_agent_t gpu_device() const { return gpu_device_; }
   hsa_queue_t* gpu_queue() { return gpu_queue_; }
+  void set_gpu_queue(hsa_queue_t* gpu_queue) { gpu_queue_ = gpu_queue; }
 
   // Return pointer to PrintfDbg
   PrintfDbg* printfDbg() const { return printfdbg_; }
@@ -430,16 +445,20 @@ class VirtualGPU : public device::VirtualDevice {
   HwQueueTracker& Barriers() { return barriers_; }
 
   Timestamp* timestamp() const { return timestamp_; }
+  amd::Command* command() const { return command_; }
 
   void* allocKernArg(size_t size, size_t alignment);
   bool isFenceDirty() const { return fence_dirty_; }
+  void setFenceDirty(bool state) { fence_dirty_ = state; }
   void HiddenHeapInit();
 
   void setLastUsedSdmaEngine(uint32_t mask) { lastUsedSdmaEngineMask_ = mask; }
   uint32_t getLastUsedSdmaEngine() const { return lastUsedSdmaEngineMask_.load(); }
   uint64_t getQueueID();
 
-  // } roc OpenCL integration
+  //! Analyzes a crashed AQL queue to find a broken AQL packet
+  void AnalyzeAqlQueue() const;
+
  private:
   //! Dispatches a barrier with blocking HSA signals
   void dispatchBlockingWait();
@@ -512,6 +531,33 @@ class VirtualGPU : public device::VirtualDevice {
   //! Resets the current queue state. Note: should be called after AQL queue becomes idle
   void ResetQueueStates();
 
+  //! Track the progress of the queue based on the last write index and completion signal
+  template <typename AqlPacket>
+  inline void TrackQueueProgress(const AqlPacket& packet, uint64_t index) {
+    // Track the progress of the current virtual queue
+    last_write_index_ = index;
+    // Update the last completion signal if the packet has one
+    if (packet.completion_signal.handle != 0) {
+      last_barrier_index_ = index;
+      last_completion_signal_ = packet.completion_signal;
+    }
+  }
+
+  //! Returns true if the queue is considered as idle. That means all submitted packets are complete.
+  //! Note: it doesn't track the state of caches
+  bool IsQueueIdle() const {
+    bool result = false;
+    // Make sure the last packet contained a completion signal
+    if (last_barrier_index_ == last_write_index_) {
+      if ((last_write_index_ == 0) && (last_completion_signal_.handle == 0)) {
+        result = true;
+      } else {
+        result = (hsa_signal_load_relaxed(last_completion_signal_) == 0);
+      }
+    }
+    return result;
+  }
+
   std::vector<amd::Memory*> pinnedMems_;   //!< Pinned memory list
 
   //! Queue state flags
@@ -528,6 +574,7 @@ class VirtualGPU : public device::VirtualDevice {
   };
 
   Timestamp* timestamp_;
+  amd::Command* command_;   //!< Current command
   hsa_agent_t gpu_device_;  //!< Physical device
   hsa_queue_t* gpu_queue_;  //!< Active queue associated with a vgpu
   hsa_barrier_and_packet_t barrier_packet_;
@@ -577,9 +624,11 @@ class VirtualGPU : public device::VirtualDevice {
   bool fence_dirty_;                    //!< Fence modified flag
 
   std::atomic<uint> lastUsedSdmaEngineMask_;     //!< Last Used SDMA Engine mask
+  uint64_t last_write_index_ = 0;       //!< The last HW queue write index for any packet
+  uint64_t last_barrier_index_ = 0;     //!< The last HW queue write index for a packet
+                                        //!< with a complition signal
+  hsa_signal_t last_completion_signal_{}; //!< The last completion signal
 
   using KernelArgImpl = device::Settings::KernelArgImpl;
-
-  amd::Command* currCmd_ = nullptr;  //!< Current command under capture
 };
 }
