@@ -524,7 +524,7 @@ int Os::systemCall(const std::string& command) {
 #if 1
   size_t len = command.size();
   char* cmd = new char[len + 1];
-  fastMemcpy(cmd, command.c_str(), len);
+  std::memcpy(cmd, command.c_str(), len);
   cmd[len] = 0;
 
   // Split the command into arguments. This is a very
@@ -681,8 +681,6 @@ uint64_t Os::xgetbv(uint32_t ecx) {
 }
 #endif  // ATI_ARCH_X86
 
-void* Os::fastMemcpy(void* dest, const void* src, size_t n) { return memcpy(dest, src, n); }
-
 uint64_t Os::offsetToEpochNanos() {
   static uint64_t offset = 0;
 
@@ -822,8 +820,8 @@ bool amd::Os::FindFileNameFromAddress(const void* image, std::string* fname_ptr,
       tokens >> permissions
              >> std::hex >> offset >> std::dec
              >> device
-             >> inode
-             >> uri_file_path;
+             >> inode;
+      std::getline(tokens >> std::ws, uri_file_path);
 
       if (inode == 0 || uri_file_path.empty()) {
         return ret_value;
@@ -962,6 +960,15 @@ void Os::CloseIpcMemory(const FileDesc desc, const void* ptr, size_t size) {
   }
 }
 
+void Os::PrintLibraryLocation() {
+  Dl_info dl_info;
+  if (dladdr(reinterpret_cast<void*>(Os::loadLibrary), &dl_info) && dl_info.dli_fname) {
+    ClPrint(amd::LOG_INFO, amd::LOG_INIT, "HIP Library Path: %s", dl_info.dli_fname);
+  } else {
+    ClPrint(amd::LOG_INFO, amd::LOG_INIT, "HIP Library Path: <unknown>");
+  }
+}
+
 }  // namespace amd
 
 #endif  // !defined(_WIN32) && !defined(__CYGWIN__)
