@@ -88,10 +88,13 @@ class Event : public RuntimeObject {
   std::atomic<CallBackEntry*> callbacks_;  //!< linked list of callback entries.
   std::atomic<int32_t> status_;            //!< current execution status.
   std::atomic_flag notified_;              //!< Command queue was notified
+
   void*  hw_event_;                        //!< HW event ID associated with SW event
   Event* notify_event_;                    //!< Notify event, which should contain HW signal
   const Device* device_;                   //!< Device, this event associated with
-  int32_t event_scope_;                    //!< 2 - system scope, 1 - device scope,
+
+  std::atomic<int32_t> event_entry_scope_; //!< Command entry scope
+                                           //!< 2 - system scope, 1 - device scope,
                                            //!< 0 - ignore, -1 - invalid
 
  protected:
@@ -210,7 +213,7 @@ class Event : public RuntimeObject {
   //! Returns the callback for this event
   const CallBackEntry* Callback() const { return callbacks_; }
 
-  // Saves HW event, associated with the current command
+  //! Saves HW event, associated with the current command
   void SetHwEvent(void* hw_event) { hw_event_ = hw_event; }
 
   //! Returns HW event, associated with the current command
@@ -219,11 +222,15 @@ class Event : public RuntimeObject {
   //! Returns notify even associated with the current command
   Event* NotifyEvent() const { return notify_event_; }
 
-  //! Get release scope of the event
-  int32_t getEventScope() const { return event_scope_; }
+  //! Get entry scope of the event
+  int32_t getCommandEntryScope() const {
+    return event_entry_scope_.load(std::memory_order_relaxed);
+  }
 
-  //! Set release scope for the event
-  void setEventScope(int32_t scope) { event_scope_ = scope; }
+  //! Set entry scope for the event
+  void setCommandEntryScope(int32_t scope) {
+    event_entry_scope_.store(scope, std::memory_order_relaxed);
+  }
 };
 
 union CopyMetadata {
@@ -1258,7 +1265,7 @@ class NDRangeKernelCommand : public Command {
   int32_t captureAndValidate();
 
   // Allocate, capture and set kernel parameters
-  int32_t AllocCaptureSetValidate(void** kernelParams, address kernArgs);
+  int32_t AllocCaptureSetValidate(void** kernelParams, address kernArgs, size_t kernArgsSize);
 };
 
 class NativeFnCommand : public Command {
