@@ -1,4 +1,4 @@
-/* Copyright (c) 2010 - 2022 Advanced Micro Devices, Inc.
+/* Copyright (c) 2010 - 2025 Advanced Micro Devices, Inc.
 
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
@@ -77,7 +77,8 @@ void HostMemoryReference::deallocateMemory(const Context& context) {
   }
 }
 
-Memory::Memory(Context& context, Type type, Flags flags, size_t size, void* svmPtr)
+Memory::Memory(Context& context, Type type, Flags flags, size_t size, void* svmPtr,
+               size_t alignment)
     : numDevices_(0),
       deviceMemories_(NULL),
       destructorCallbacks_(NULL),
@@ -96,7 +97,8 @@ Memory::Memory(Context& context, Type type, Flags flags, size_t size, void* svmP
       svmHostAddress_(svmPtr),
       resOffset_(0),
       flagsEx_(0),
-      lockMemoryOps_("Memory Ops Lock", true) {
+      lockMemoryOps_(true),
+      alignment_(alignment) /* Memory Ops Lock */ {
   svmPtrCommited_ = (flags & CL_MEM_SVM_FINE_GRAIN_BUFFER) ? true : false;
   canBeCached_ = true;
 }
@@ -120,7 +122,7 @@ Memory::Memory(Memory& parent, Flags flags, size_t origin, size_t size, Type typ
       svmHostAddress_(parent.getSvmPtr()),
       resOffset_(0),
       flagsEx_(0),
-      lockMemoryOps_("Memory Ops Lock", true) {
+      lockMemoryOps_(true)  /* Memory Ops Lock */ {
   svmPtrCommited_ = parent.isSvmPtrCommited();
   canBeCached_ = true;
   parent_->retain();
@@ -192,12 +194,12 @@ void Memory::operator delete(void* p, const Context& context) { Memory::operator
 
 void Memory::addSubBuffer(Memory* view) {
   amd::ScopedLock lock(lockMemoryOps());
-  subBuffers_.push_back(view);
+  subBuffers_.emplace(view);
 }
 
 void Memory::removeSubBuffer(Memory* view) {
   amd::ScopedLock lock(lockMemoryOps());
-  subBuffers_.remove(view);
+  subBuffers_.erase(view);
 }
 
 bool Memory::allocHostMemory(void* initFrom, bool allocHostMem, bool forceCopy) {
@@ -414,6 +416,14 @@ device::Memory* Memory::getDeviceMemory(const Device& dev, bool alloc) {
 
 // ================================================================================================
 Memory::~Memory() {
+
+  if (ipcShared()) {
+    amd::MemObjMap::RemoveIpcHandleMemObj(this);
+    auto device = context_().devices()[0];
+    if (device != nullptr) {
+      device->IpcDetach(this);
+    }
+  }
   // For_each destructor callback:
   DestructorCallBackEntry* entry;
   for (entry = destructorCallbacks_; entry != nullptr; entry = entry->next_) {
@@ -530,16 +540,22 @@ void Memory::commitSvmMemory() {
   ScopedLock lock(lockMemoryOps_);
   // if VRAM is visible for host, it is not necessary to mmap again
   if (!svmPtrCommited_ && !largeBarSystem_) {
-    amd::Os::commitMemory(svmHostAddress_, size_, amd::Os::MEM_PROT_RW);
-    svmPtrCommited_ = true;
+    if (amd::Os::commitMemory(svmHostAddress_, size_, amd::Os::MEM_PROT_RW)) {
+      svmPtrCommited_ = true;
+    } else {
+      LogPrintfError("Mem Map failed for the host address 0x%x", svmHostAddress_);
+    }
   }
 }
 
 void Memory::uncommitSvmMemory() {
   ScopedLock lock(lockMemoryOps_);
   if (svmPtrCommited_ && !(flags_ & CL_MEM_SVM_FINE_GRAIN_BUFFER)) {
-    amd::Os::uncommitMemory(svmHostAddress_, size_);
-    svmPtrCommited_ = false;
+    if (amd::Os::uncommitMemory(svmHostAddress_, size_)) {
+      svmPtrCommited_ = false;
+    } else {
+      LogPrintfError("Mem Unmap failed for the host address 0x%x", svmHostAddress_);
+    }
   }
 }
 
@@ -548,6 +564,14 @@ Device* Memory::GetDeviceById() {
   return getContext().devices()[device_idx];
 }
 
+// =================================================================================================
+bool Memory::ValidateMemAccess(const Device& dev, bool read_write) {
+  if (flags_ & CL_MEM_VA_RANGE_AMD) {
+    return dev.ValidateMemAccess(*this, read_write);
+  }
+  return true;
+}
+
 void Buffer::initDeviceMemory() {
   deviceMemories_ = reinterpret_cast<DeviceMemory*>(reinterpret_cast<char*>(this) + sizeof(Buffer));
   memset(deviceMemories_, 0, NumDevicesWithP2P() * sizeof(DeviceMemory));
@@ -661,10 +685,6 @@ bool Image::validateDimensions(const std::vector<amd::Device*>& devices, cl_mem_
       }
     // Fall through...
     case CL_MEM_OBJECT_IMAGE2D:
-      if ((width == 0) || (height == 0)) {
-        DevLogPrintfError("Invalid dimensions width: %u height: %u \n", width, height);
-        return false;
-      }
       for (const auto dev : devices) {
         if ((dev->info().image2DMaxHeight_ >= height) && (dev->info().image2DMaxWidth_ >= width)) {
           return true;
@@ -700,10 +720,6 @@ bool Image::validateDimensions(const std::vector<amd::Device*>& devices, cl_mem_
       }
       break;
     case CL_MEM_OBJECT_IMAGE1D_BUFFER:
-      if (width == 0) {
-        DevLogError("Invalid dimension \n");
-        return false;
-      }
       for (const auto& dev : devices) {
         if (dev->info().imageMaxBufferSize_ >= width) {
           return true;
@@ -1286,6 +1302,15 @@ void Image::copyToBackingStore(void* initFrom) {
   }
 }
 
+template <typename In, typename Out> static Out interpret_value(In in) {
+  static_assert(sizeof(In) == sizeof(Out));
+  union {
+    In in_val;
+    Out out_val;
+  } u{in};
+  return u.out_val;
+}
+
 static int round_to_even(float v) {
   // clamp overflow
   if (v >= -(float)std::numeric_limits<int>::min()) {
@@ -1297,8 +1322,8 @@ static int round_to_even(float v) {
   static const unsigned int magic[2] = {0x4b000000u, 0xcb000000u};
 
   // round fractional values to integer value
-  if (fabsf(v) < *reinterpret_cast<const float*>(&magic[0])) {
-    float magicVal = *reinterpret_cast<const float*>(&magic[v < 0.0f]);
+  if (fabsf(v) < interpret_value<unsigned, float>(magic[0])) {
+    float magicVal = interpret_value<unsigned, float>(magic[v < 0.0f]);
     v += magicVal;
     v -= magicVal;
   }
@@ -1323,21 +1348,21 @@ static uint16_t float2half_rtz(float f) {
   }
   int values[5] = {0x47800000, 0x33800000, 0x38800000, 0x4b800000, 0x7f800000};
   // overflow
-  if (x >= *reinterpret_cast<float*>(&values[0])) {
-    if (x == *reinterpret_cast<float*>(&values[4])) {
+  if (x >= interpret_value<int, float>(values[0])) {
+    if (x == interpret_value<int, float>(values[4])) {
       return 0x7c00 | sign;
     }
     return 0x7bff | sign;
   }
 
   // underflow
-  if (x < *reinterpret_cast<float*>(&values[1])) {
+  if (x < interpret_value<int, float>(values[1])) {
     return sign;  // The halfway case can return 0x0001 or 0. 0 is even.
   }
 
   // half denormal
-  if (x < *reinterpret_cast<float*>(&values[2])) {
-    x *= *reinterpret_cast<float*>(&values[3]);
+  if (x < interpret_value<int, float>(values[2])) {
+    x *= interpret_value<int, float>(values[3]);
     return static_cast<uint16_t>((int)x | sign);
   }
 
